@@ -11,5 +11,6 @@ class EchoFactory(protocol.Factory):
     def buildProtocol(self, addr):
         return Echo()
 
-endpoints.serverFromString(reactor, "onion:1234").listen(EchoFactory()).addCallback(lambda x: print(x.getHost()))
+d = endpoints.serverFromString(reactor, "onion:1234").listen(EchoFactory())
+d.addCallback(lambda x: print(x.getHost()))
 reactor.run()
