@@ -2,14 +2,25 @@ import datetime
 import time
 from twisted.trial import unittest
 from twisted.internet import defer
+from twisted.python.failure import Failure
 from zope.interface import implements
 
-from txtorcon import Circuit, Stream, TorControlProtocol, TorState
-from txtorcon.interface import IRouterContainer, ICircuitListener, ICircuitContainer, CircuitListenerMixin
+from txtorcon import Circuit
+from txtorcon import Stream
+from txtorcon import TorControlProtocol
+from txtorcon import TorState
+from txtorcon import Router
+from txtorcon.interface import IRouterContainer
+from txtorcon.interface import ICircuitListener
+from txtorcon.interface import ICircuitContainer
+from txtorcon.interface import CircuitListenerMixin
+from txtorcon.interface import ITorControlProtocol
 
 
 class FakeTorController(object):
-    implements(IRouterContainer, ICircuitListener, ICircuitContainer)
+    implements(IRouterContainer, ICircuitListener, ICircuitContainer, ITorControlProtocol)
+
+    post_bootstrap = defer.Deferred()
 
     def __init__(self):
         self.routers = {}
@@ -105,9 +116,9 @@ class CircuitTests(unittest.TestCase):
         from zope.interface.verify import verifyObject
         self.assertTrue(verifyObject(ICircuitListener, listener))
 
-        ## call all the methods with None for each arg. This is mostly
-        ## just to gratuitously increase test coverage, but also
-        ## serves to ensure these methods don't just blow up
+        # call all the methods with None for each arg. This is mostly
+        # just to gratuitously increase test coverage, but also
+        # serves to ensure these methods don't just blow up
         for (methodname, desc) in ICircuitListener.namesAndDescriptions():
             method = getattr(listener, methodname)
             args = [None] * len(desc.positional)
@@ -115,10 +126,13 @@ class CircuitTests(unittest.TestCase):
 
     def test_unlisten(self):
         tor = FakeTorController()
-        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = FakeRouter(
+            '$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a'
+        )
 
         circuit = Circuit(tor)
         circuit.listen(tor)
+        circuit.listen(tor)
         circuit.update('1 LAUNCHED PURPOSE=GENERAL'.split())
         circuit.unlisten(tor)
         circuit.update('1 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris PURPOSE=GENERAL'.split())
@@ -126,6 +140,7 @@ class CircuitTests(unittest.TestCase):
         self.assertTrue(1 in tor.circuits)
         self.assertEqual(len(tor.extend), 0)
         self.assertEqual(1, len(circuit.path))
+        self.assertEqual(0, len(circuit.listeners))
 
     def test_path_update(self):
         cp = TorControlProtocol()
@@ -133,7 +148,10 @@ class CircuitTests(unittest.TestCase):
         circuit = Circuit(state)
         circuit.update('1 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris PURPOSE=GENERAL'.split())
         self.assertEqual(1, len(circuit.path))
-        self.assertEqual('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', circuit.path[0].id_hex)
+        self.assertEqual(
+            '$E11D2B2269CC25E67CA6C9FB5843497539A74FD0',
+            circuit.path[0].id_hex
+        )
         self.assertEqual('eris', circuit.path[0].name)
 
     def test_wrong_update(self):
@@ -141,7 +159,11 @@ class CircuitTests(unittest.TestCase):
         circuit = Circuit(tor)
         circuit.listen(tor)
         circuit.update('1 LAUNCHED PURPOSE=GENERAL'.split())
-        self.assertRaises(Exception, circuit.update, '2 LAUNCHED PURPOSE=GENERAL'.split())
+        self.assertRaises(
+            Exception,
+            circuit.update,
+            '2 LAUNCHED PURPOSE=GENERAL'.split()
+        )
 
     def test_closed_remaining_streams(self):
         tor = FakeTorController()
@@ -162,16 +184,25 @@ class CircuitTests(unittest.TestCase):
         tor = FakeTorController()
         circuit = Circuit(tor)
         circuit.listen(tor)
-        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
-        tor.routers['$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5'] = FakeRouter('$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5', 'b')
-        tor.routers['$253DFF1838A2B7782BE7735F74E50090D46CA1BC'] = FakeRouter('$253DFF1838A2B7782BE7735F74E50090D46CA1BC', 'c')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = FakeRouter(
+            '$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a'
+        )
+        tor.routers['$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5'] = FakeRouter(
+            '$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5', 'b'
+        )
+        tor.routers['$253DFF1838A2B7782BE7735F74E50090D46CA1BC'] = FakeRouter(
+            '$253DFF1838A2B7782BE7735F74E50090D46CA1BC', 'c'
+        )
 
         for ex in examples[:-1]:
             circuit.update(ex.split()[1:])
             self.assertEqual(circuit.state, ex.split()[2])
             self.assertEqual(circuit.purpose, 'GENERAL')
             if '$' in ex:
-                self.assertEqual(len(circuit.path), len(ex.split()[3].split(',')))
+                self.assertEqual(
+                    len(circuit.path),
+                    len(ex.split()[3].split(','))
+                )
                 for (r, p) in zip(ex.split()[3].split(','), circuit.path):
                     d = r.split('=')[0]
                     self.assertEqual(d, p.hash)
@@ -207,7 +238,8 @@ class CircuitTests(unittest.TestCase):
 
     def test_extends_no_path(self):
         '''
-        without connectivity, it seems you get EXTENDS messages with no path update.
+        without connectivity, it seems you get EXTENDS messages with no
+        path update.
         '''
         tor = FakeTorController()
         circuit = Circuit(tor)
@@ -225,6 +257,9 @@ class CircuitTests(unittest.TestCase):
         circuit = Circuit(tor)
         circuit.id = 1
         str(circuit)
+        router = Router(tor)
+        circuit.path.append(router)
+        str(circuit)
 
     def test_failed_reason(self):
         tor = FakeTorController()
@@ -259,7 +294,8 @@ class CircuitTests(unittest.TestCase):
         # CLOSECIRCUIT call (see close_circuit() in FakeTorController
         # above) however the circuit isn't "really" closed yet...
         self.assertTrue(not d.called)
-        ## not unit-test-y? shouldn't probably delve into internals I suppose...
+        # not unit-test-y? shouldn't probably delve into internals I
+        # suppose...
         self.assertTrue(circuit._closing_deferred is not None)
 
         # simulate that Tor has really closed the circuit for us
@@ -267,5 +303,62 @@ class CircuitTests(unittest.TestCase):
         circuit.update('123 CLOSED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL REASON=FINISHED'.split())
 
         # confirm that our circuit callback has been triggered already
-        self.assertRaises(defer.AlreadyCalledError, d.callback, "should have been called already")
+        self.assertRaises(
+            defer.AlreadyCalledError,
+            d.callback,
+            "should have been called already"
+        )
         return d
+
+    def test_is_built(self):
+        tor = FakeTorController()
+        a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        b = FakeRouter('$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5', 'b')
+        c = FakeRouter('$253DFF1838A2B7782BE7735F74E50090D46CA1BC', 'c')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = a
+        tor.routers['$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5'] = b
+        tor.routers['$253DFF1838A2B7782BE7735F74E50090D46CA1BC'] = c
+
+        circuit = Circuit(tor)
+        circuit.listen(tor)
+
+        circuit.update('123 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL'.split())
+        built0 = circuit.is_built
+        built1 = circuit.when_built()
+
+        self.assertTrue(built0 is not built1)
+        self.assertFalse(built0.called)
+        self.assertFalse(built1.called)
+
+        circuit.update('123 BUILT $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris,$50DD343021E509EB3A5A7FD0D8A4F8364AFBDCB5=venus,$253DFF1838A2B7782BE7735F74E50090D46CA1BC=chomsky PURPOSE=GENERAL'.split())
+
+        # create callback when we're alread in BUILT; should be
+        # callback'd already
+        built2 = circuit.when_built()
+
+        self.assertTrue(built2 is not built1)
+        self.assertTrue(built2 is not built0)
+        self.assertTrue(built0.called)
+        self.assertTrue(built1.called)
+        self.assertTrue(built2.called)
+        self.assertTrue(built0.result == circuit)
+        self.assertTrue(built1.result == circuit)
+        self.assertTrue(built2.result == circuit)
+
+    def test_is_built_errback(self):
+        tor = FakeTorController()
+        a = FakeRouter('$E11D2B2269CC25E67CA6C9FB5843497539A74FD0', 'a')
+        tor.routers['$E11D2B2269CC25E67CA6C9FB5843497539A74FD0'] = a
+
+        state = TorState(tor)
+        circuit = Circuit(tor)
+        circuit.listen(tor)
+
+        circuit.update('123 EXTENDED $E11D2B2269CC25E67CA6C9FB5843497539A74FD0=eris PURPOSE=GENERAL'.split())
+        state.circuit_new(circuit)
+        d = circuit.when_built()
+
+        state.circuit_closed(circuit)
+
+        self.assertTrue(d.called)
+        self.assertTrue(isinstance(d.result, Failure))
