@@ -14,6 +14,7 @@ from twisted.internet.endpoints import TCP4ServerEndpoint
 from twisted.internet.endpoints import serverFromString
 from twisted.internet.endpoints import clientFromString
 from twisted.python.failure import Failure
+from twisted.internet.error import ConnectionRefusedError
 from twisted.internet.interfaces import IReactorCore
 from twisted.internet.interfaces import IProtocolFactory
 from twisted.internet.interfaces import IProtocol
@@ -26,16 +27,21 @@ from txtorcon import ITorControlProtocol
 from txtorcon import TorConfig
 from txtorcon import launch_tor
 from txtorcon import TCPHiddenServiceEndpoint
+from txtorcon import TorClientEndpoint
 from txtorcon import TorNotFound
 from txtorcon import TCPHiddenServiceEndpointParser
 from txtorcon import IProgressProvider
 from txtorcon import TorOnionAddress
 from txtorcon.util import NoOpProtocolFactory
 from txtorcon.endpoints import get_global_tor                       # FIXME
+from txtorcon.endpoints import default_tcp4_endpoint_generator
 
 import util
 
 
+connectionRefusedFailure = Failure(ConnectionRefusedError())
+
+
 class EndpointTests(unittest.TestCase):
 
     def setUp(self):
@@ -45,10 +51,21 @@ class EndpointTests(unittest.TestCase):
         endpoints._global_tor_lock = defer.DeferredLock()
         self.reactor = FakeReactorTcp(self)
         self.protocol = FakeControlProtocol([])
+        self.protocol.event_happened('INFO', 'something craaaaaaazy')
+        self.protocol.event_happened(
+            'INFO',
+            'connection_dir_client_reached_eof(): Uploaded rendezvous '
+            'descriptor (status 200 ("Service descriptor (v2) stored"))'
+        )
         self.config = TorConfig(self.protocol)
-        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual')
+        self.protocol.answers.append(
+            'config/names=\nHiddenServiceOptions Virtual'
+        )
         self.protocol.answers.append('HiddenServiceOptions')
-        self.patcher = patch('txtorcon.torconfig.find_tor_binary', return_value='/not/tor')
+        self.patcher = patch(
+            'txtorcon.torconfig.find_tor_binary',
+            return_value='/not/tor'
+        )
         self.patcher.start()
 
     def tearDown(self):
@@ -60,12 +77,18 @@ class EndpointTests(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_global_tor(self):
-        config = yield get_global_tor(Mock(), _tor_launcher=lambda x, y, z: True)
+        config = yield get_global_tor(
+            Mock(),
+            _tor_launcher=lambda x, y, z: True
+        )
         self.assertEqual(0, config.SOCKSPort)
 
     @defer.inlineCallbacks
     def test_global_tor_error(self):
-        config0 = yield get_global_tor(Mock(), _tor_launcher=lambda x, y, z: True)
+        config0 = yield get_global_tor(
+            Mock(),
+            _tor_launcher=lambda x, y, z: True
+        )
         # now if we specify a control_port it should be an error since
         # the above should have launched one.
         try:
@@ -92,7 +115,7 @@ class EndpointTests(unittest.TestCase):
         endpoints.launch_tor = m
         ep = yield TCPHiddenServiceEndpoint.private_tor(Mock(), 80,
                                                         control_port=1234)
-        m.assert_called()
+        self.assertTrue(m.called)
 
     @defer.inlineCallbacks
     def test_private_tor_no_control_port(self):
@@ -100,7 +123,7 @@ class EndpointTests(unittest.TestCase):
         from txtorcon import endpoints
         endpoints.launch_tor = m
         ep = yield TCPHiddenServiceEndpoint.private_tor(Mock(), 80)
-        m.assert_called()
+        self.assertTrue(m.called)
 
     @defer.inlineCallbacks
     def test_system_tor(self):
@@ -113,9 +136,12 @@ class EndpointTests(unittest.TestCase):
             def bam(*args, **kw):
                 return self.protocol
             return bam
-        with patch('txtorcon.endpoints.launch_tor') as m:
+        with patch('txtorcon.endpoints.launch_tor') as launch_mock:
             with patch('txtorcon.endpoints.build_tor_connection', new_callable=boom) as btc:
-                client = clientFromString(self.reactor, "tcp:host=localhost:port=9050")
+                client = clientFromString(
+                    self.reactor,
+                    "tcp:host=localhost:port=9050"
+                )
                 ep = yield TCPHiddenServiceEndpoint.system_tor(self.reactor,
                                                                client, 80)
                 port = yield ep.listen(NoOpProtocolFactory())
@@ -125,7 +151,9 @@ class EndpointTests(unittest.TestCase):
                 port.startListening()
                 str(port)
                 port.tor_config
-                m.assert_called()
+                # system_tor should be connecting to a running one,
+                # *not* launching a new one.
+                self.assertFalse(launch_mock.called)
 
     @defer.inlineCallbacks
     def test_basic(self):
@@ -159,7 +187,41 @@ class EndpointTests(unittest.TestCase):
         args = (50, "blarg", "Doing that thing we talked about.")
         # kind-of cheating, test-wise?
         ep._tor_progress_update(*args)
-        ding.assert_called_with(*args)
+        self.assertTrue(ding.called_with(*args))
+
+    @patch('txtorcon.endpoints.launch_tor')
+    def test_progress_updates_private_tor(self, tor):
+        ep = TCPHiddenServiceEndpoint.private_tor(self.reactor, 1234)
+        tor.call_args[1]['progress_updates'](40, 'FOO', 'foo to the bar')
+        return ep
+
+    def __test_progress_updates_system_tor(self):
+        ep = TCPHiddenServiceEndpoint.system_tor(self.reactor, 1234)
+        ep._tor_progress_update(40, "FOO", "foo to bar")
+        return ep
+
+    @patch('txtorcon.endpoints.get_global_tor')
+    def test_progress_updates_global_tor(self, tor):
+        ep = TCPHiddenServiceEndpoint.global_tor(self.reactor, 1234)
+        tor.call_args[1]['progress_updates'](40, 'FOO', 'foo to the bar')
+        return ep
+
+    def test_hiddenservice_key_unfound(self):
+        ep = TCPHiddenServiceEndpoint.private_tor(
+            self.reactor,
+            1234,
+            hidden_service_dir='/dev/null'
+        )
+
+        # FIXME Mock() should work somehow for this, but I couldn't
+        # make it "go"
+        class Blam(object):
+            @property
+            def private_key(self):
+                raise IOError("blam")
+        ep.hiddenservice = Blam()
+        self.assertEqual(ep.onion_private_key, None)
+        return ep
 
     def test_multiple_listen(self):
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
@@ -180,37 +242,31 @@ class EndpointTests(unittest.TestCase):
 
         def check(arg):
             self.assertEqual('127.0.0.1', ep.tcp_endpoint._interface)
+            self.assertEqual(len(self.config.HiddenServices), 1)
         d0.addCallback(check).addErrback(self.fail)
         return d0
 
     def test_already_bootstrapped(self):
         self.config.bootstrap()
-
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(NoOpProtocolFactory())
         return d
 
     @defer.inlineCallbacks
     def test_explicit_data_dir(self):
-        config = TorConfig()
-        td = tempfile.mkdtemp()
-        ep = TCPHiddenServiceEndpoint(self.reactor, config, 123, td)
-
-        # fake out some things so we don't actually have to launch + bootstrap
-        class FakeTorProcessProtocol(object):
-            tor_protocol = self.reactor.protocol
-        process = FakeTorProcessProtocol()
-        ep._launch_tor = Mock(return_value=process)
-        config._update_proto(Mock())
-        config.bootstrap()
-        yield config.post_bootstrap
+        config = TorConfig(self.protocol)
+        ep = TCPHiddenServiceEndpoint(self.reactor, config, 123, '/dev/null')
 
         # make sure listen() correctly configures our hidden-serivce
         # with the explicit directory we passed in above
-        port = yield ep.listen(NoOpProtocolFactory())
+        d = ep.listen(NoOpProtocolFactory())
+
+        def foo(fail):
+            print "ERROR", fail
+        d.addErrback(foo)
+        port = yield d
         self.assertEqual(1, len(config.HiddenServices))
-        self.assertEqual(config.HiddenServices[0].dir, td)
-        shutil.rmtree(td)
+        self.assertEqual(config.HiddenServices[0].dir, '/dev/null')
 
     def test_failure(self):
         self.reactor.failures = 1
@@ -225,14 +281,20 @@ class EndpointTests(unittest.TestCase):
         return None
 
     def test_parse_via_plugin(self):
-        # make sure we have a valid thing from get_global_tor without actually launching tor
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
         config = TorConfig()
         config.post_bootstrap = defer.succeed(config)
         from txtorcon import torconfig
         torconfig._global_tor_config = None
-        get_global_tor(self.reactor,
-                       _tor_launcher=lambda react, config, prog: defer.succeed(config))
-        ep = serverFromString(self.reactor, 'onion:88:localPort=1234:hiddenServiceDir=/foo/bar')
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, prog: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:hiddenServiceDir=/foo/bar'
+        )
         self.assertEqual(ep.public_port, 88)
         self.assertEqual(ep.local_port, 1234)
         self.assertEqual(ep.hidden_service_dir, '/foo/bar')
@@ -241,30 +303,42 @@ class EndpointTests(unittest.TestCase):
         # this makes sure we expand users and symlinks in
         # hiddenServiceDir args. see Issue #77
 
-        # make sure we have a valid thing from get_global_tor without actually launching tor
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
         config = TorConfig()
         config.post_bootstrap = defer.succeed(config)
         from txtorcon import torconfig
         torconfig._global_tor_config = None
-        get_global_tor(self.reactor,
-                       _tor_launcher=lambda react, config, prog: defer.succeed(config))
-        ep = serverFromString(self.reactor, 'onion:88:localPort=1234:hiddenServiceDir=~/blam/blarg')
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, prog: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:hiddenServiceDir=~/blam/blarg'
+        )
         # would be nice to have a fixed path here, but then would have
         # to run as a known user :/
         # maybe using the docker stuff to run integration tests better here?
-        self.assertEqual(os.path.expanduser('~/blam/blarg'), ep.hidden_service_dir)
+        self.assertEqual(
+            os.path.expanduser('~/blam/blarg'),
+            ep.hidden_service_dir
+        )
 
     def test_parse_relative_path(self):
         # this makes sure we convert a relative path to absolute
         # hiddenServiceDir args. see Issue #77
 
-        # make sure we have a valid thing from get_global_tor without actually launching tor
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
         config = TorConfig()
         config.post_bootstrap = defer.succeed(config)
         from txtorcon import torconfig
         torconfig._global_tor_config = None
-        get_global_tor(self.reactor,
-                       _tor_launcher=lambda react, config, prog: defer.succeed(config))
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, prog: defer.succeed(config)
+        )
 
         orig = os.path.realpath('.')
         try:
@@ -272,10 +346,17 @@ class EndpointTests(unittest.TestCase):
                 t = str(t)
                 os.chdir(t)
                 os.mkdir(os.path.join(t, 'foo'))
-                os.mkdir(os.path.join(t, 'foo', 'blam'))
+                hsdir = os.path.join(t, 'foo', 'blam')
+                os.mkdir(hsdir)
 
-                ep = serverFromString(self.reactor, 'onion:88:localPort=1234:hiddenServiceDir=foo/blam')
-                self.assertEqual(os.path.join(t, 'foo', 'blam'), ep.hidden_service_dir)
+                ep = serverFromString(
+                    self.reactor,
+                    'onion:88:localPort=1234:hiddenServiceDir=foo/blam'
+                )
+                self.assertEqual(
+                    os.path.realpath(hsdir),
+                    ep.hidden_service_dir
+                )
 
         finally:
             os.chdir(orig)
@@ -328,7 +409,10 @@ class EndpointLaunchTests(unittest.TestCase):
         """
 
         reactor = proto_helpers.MemoryReactor()
-        ep = serverFromString(reactor, 'onion:8888:controlPort=9055:localPort=1234')
+        ep = serverFromString(
+            reactor,
+            'onion:8888:controlPort=9055:localPort=1234'
+        )
         r = yield ep.listen(NoOpProtocolFactory())
         self.assertEqual(global_tor.call_count, 0)
         self.assertEqual(private_tor.call_count, 0)
@@ -399,8 +483,8 @@ def port_generator():
         yield x
 
 
-from test_torconfig import FakeReactor  # FIXME
-from test_torconfig import FakeProcessTransport  # FIXME
+from test_torconfig import FakeReactor  # FIXME put in util or something?
+from test_torconfig import FakeProcessTransport  # FIXME importing from other test sucks
 from test_torconfig import FakeControlProtocol  # FIXME
 
 
@@ -438,13 +522,160 @@ class FakeReactorTcp(FakeReactor):
 
     def connectTCP(self, host, port, factory, timeout, bindAddress):
         '''should return IConnector'''
-#        print "CONN", host, port, factory
-#        print dir(factory)
-#        print "XX", factory
-        r = tcp.Connector(host, port, factory, timeout, bindAddress, reactor=self)
-#        print dir(r)
+        r = tcp.Connector(
+            host, port, factory, timeout,
+            bindAddress, reactor=self
+        )
 
         def blam(*args):
             print "BLAAAAAM", args
         r.connect = blam
         return r
+
+
+class FakeTorSocksEndpoint(object):
+    def __init__(self, *args, **kw):
+        self.host = args[1]
+        self.port = args[2]
+        self.transport = None
+
+        self.failure = kw.get('failure', None)
+        self.acceptPort = kw.get('acceptPort', None)
+
+    def connect(self, fac):
+        self.factory = fac
+        if self.acceptPort:
+            if self.port != self.acceptPort:
+                return defer.fail(self.failure)
+        else:
+            if self.failure:
+                return defer.fail(self.failure)
+        self.proto = fac.buildProtocol(None)
+        transport = proto_helpers.StringTransport()
+        self.proto.makeConnection(transport)
+        self.transport = transport
+        return defer.succeed(self.proto)
+
+
+class TestTorClientEndpoint(unittest.TestCase):
+
+    def test_client_connection_failed(self):
+        """
+        This test is equivalent to txsocksx's
+        TestSOCKS4ClientEndpoint.test_clientConnectionFailed
+        """
+        def FailTorSocksEndpointGenerator(*args, **kw):
+            kw['failure'] = connectionRefusedFailure
+            return FakeTorSocksEndpoint(*args, **kw)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=FailTorSocksEndpointGenerator)
+        d = endpoint.connect(None)
+        return self.assertFailure(d, ConnectionRefusedError)
+
+    def test_client_connection_failed_user_password(self):
+        """
+        Same as above, but with a username/password.
+        """
+        def FailTorSocksEndpointGenerator(*args, **kw):
+            kw['failure'] = connectionRefusedFailure
+            return FakeTorSocksEndpoint(*args, **kw)
+        endpoint = TorClientEndpoint(
+            'invalid host', 0,
+            socks_username='billy', socks_password='s333cure',
+            _proxy_endpoint_generator=FailTorSocksEndpointGenerator)
+        d = endpoint.connect(None)
+        return self.assertFailure(d, ConnectionRefusedError)
+
+    def test_default_generator(self):
+        # just ensuring the default generator doesn't blow updoesn't blow up
+        default_tcp4_endpoint_generator(None, 'foo.bar', 1234)
+
+    def test_no_host(self):
+        self.assertRaises(
+            ValueError,
+            TorClientEndpoint, None, None
+        )
+
+    def test_parser_basic(self):
+        ep = clientFromString(None, 'tor:host=timaq4ygg2iegci7.onion:port=80:socksPort=9050')
+
+        self.assertEqual(ep.host, 'timaq4ygg2iegci7.onion')
+        self.assertEqual(ep.port, 80)
+        self.assertEqual(ep.socks_port, 9050)
+
+    def test_parser_user_password(self):
+        epstring = 'tor:host=torproject.org:port=443' + \
+                   ':socksUsername=foo:socksPassword=bar'
+        ep = clientFromString(None, epstring)
+
+        self.assertEqual(ep.host, 'torproject.org')
+        self.assertEqual(ep.port, 443)
+        self.assertEqual(ep.socks_username, 'foo')
+        self.assertEqual(ep.socks_password, 'bar')
+
+    def test_default_factory(self):
+        """
+        This test is equivalent to txsocksx's TestSOCKS5ClientEndpoint.test_defaultFactory
+        """
+        def TorSocksEndpointGenerator(*args, **kw):
+            return FakeTorSocksEndpoint(*args, **kw)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
+        endpoint.connect(None)
+        self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+
+    def test_good_port_retry(self):
+        """
+        This tests that our Tor client endpoint retry logic works correctly.
+        We create a proxy endpoint that fires a connectionRefusedFailure
+        unless the connecting port matches. We attempt to connect with the
+        proxy endpoint for each port that the Tor client endpoint will try.
+        """
+        success_ports = TorClientEndpoint.socks_ports_to_try
+        for port in success_ports:
+            def TorSocksEndpointGenerator(*args, **kw):
+                kw['acceptPort'] = port
+                kw['failure'] = connectionRefusedFailure
+                return FakeTorSocksEndpoint(*args, **kw)
+            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
+            endpoint.connect(None)
+            self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+
+    def test_bad_port_retry(self):
+        """
+        This tests failure to connect to the ports on the "try" list.
+        """
+        fail_ports = [1984, 666]
+        for port in fail_ports:
+            def TorSocksEndpointGenerator(*args, **kw):
+                kw['acceptPort'] = port
+                kw['failure'] = connectionRefusedFailure
+                return FakeTorSocksEndpoint(*args, **kw)
+            endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator)
+            d = endpoint.connect(None)
+            return self.assertFailure(d, ConnectionRefusedError)
+
+    def test_good_no_guess_socks_port(self):
+        """
+        This tests that if a SOCKS port is specified, we *only* attempt to
+        connect to that SOCKS port.
+        """
+        def TorSocksEndpointGenerator(*args, **kw):
+            kw['acceptPort'] = 6669
+            kw['failure'] = connectionRefusedFailure
+            return FakeTorSocksEndpoint(*args, **kw)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator, socks_port=6669)
+        endpoint.connect(None)
+        self.assertEqual(endpoint.tor_socks_endpoint.transport.value(), '\x05\x01\x00')
+
+    def test_bad_no_guess_socks_port(self):
+        """
+        This tests that are connection fails if we try to connect to an unavailable
+        specified SOCKS port... even if there is a valid SOCKS port listening on
+        the socks_ports_to_try list.
+        """
+        def TorSocksEndpointGenerator(*args, **kw):
+            kw['acceptPort'] = 9050
+            kw['failure'] = connectionRefusedFailure
+            return FakeTorSocksEndpoint(*args, **kw)
+        endpoint = TorClientEndpoint('', 0, _proxy_endpoint_generator=TorSocksEndpointGenerator, socks_port=6669)
+        d = endpoint.connect(None)
+        self.assertFailure(d, ConnectionRefusedError)
