@@ -1,5 +1,7 @@
 from __future__ import with_statement
 
+from os.path import exists
+
 from twisted.python import log, failure
 from twisted.trial import unittest
 from twisted.test import proto_helpers
@@ -25,8 +27,9 @@ class CallbackChecker:
         v = args[0]
         if v != self.expected_value:
             print "WRONG"
-            raise RuntimeError('Expected "%s" but got "%s"' % (self.expected_value, v))
-        ##print "got correct value",v
+            raise RuntimeError(
+                'Expected "%s" but got "%s"' % (self.expected_value, v)
+            )
         self.called_back = True
         return v
 
@@ -53,7 +56,7 @@ class LogicTests(unittest.TestCase):
         self.assertTrue(d.called)
         self.assertTrue(d.result)
         self.assertTrue('even number' in d.result.getErrorMessage())
-        ## ignore the error so trial doesn't get unhappy
+        # ignore the error so trial doesn't get unhappy
         d.addErrback(lambda foo: True)
         return d
 
@@ -84,7 +87,10 @@ class AuthenticationTests(unittest.TestCase):
         self.send('250-VERSION Tor="0.2.2.34"')
         self.send('250 OK')
 
-        self.assertEqual(self.transport.value(), 'AUTHENTICATE %s\r\n' % cookie_data.encode("hex"))
+        self.assertEqual(
+            self.transport.value(),
+            'AUTHENTICATE %s\r\n' % cookie_data.encode("hex")
+        )
 
     def test_authenticate_password(self):
         self.protocol.password_function = lambda: 'foo'
@@ -109,13 +115,29 @@ class AuthenticationTests(unittest.TestCase):
         self.send('250-VERSION Tor="0.2.2.34"')
         self.send('250 OK')
 
-        ## make sure we haven't tried to authenticate before getting
-        ## the password callback
+        # make sure we haven't tried to authenticate before getting
+        # the password callback
         self.assertEqual(self.transport.value(), '')
         d.callback('foo')
 
-        ## now make sure we DID try to authenticate
-        self.assertEqual(self.transport.value(), 'AUTHENTICATE %s\r\n' % "foo".encode("hex"))
+        # now make sure we DID try to authenticate
+        self.assertEqual(
+            self.transport.value(),
+            'AUTHENTICATE %s\r\n' % "foo".encode("hex")
+        )
+
+    def test_authenticate_password_deferred_but_no_password(self):
+        d = defer.Deferred()
+        self.protocol.password_function = lambda: d
+        self.protocol.makeConnection(self.transport)
+        self.assertEqual(self.transport.value(), 'PROTOCOLINFO 1\r\n')
+        self.transport.clear()
+        self.send('250-PROTOCOLINFO 1')
+        self.send('250-AUTH METHODS=HASHEDPASSWORD')
+        self.send('250-VERSION Tor="0.2.2.34"')
+        self.send('250 OK')
+        d.callback(None)
+        return self.assertFailure(self.protocol.post_bootstrap, RuntimeError)
 
     def confirmAuthFailed(self, *args):
         self.auth_failed = True
@@ -141,7 +163,7 @@ class DisconnectionTests(unittest.TestCase):
         self.protocol.connectionMade = lambda: None
         self.transport = proto_helpers.StringTransportWithDisconnection()
         self.protocol.makeConnection(self.transport)
-        ## why doesn't makeConnection do this?
+        # why doesn't makeConnection do this?
         self.transport.protocol = self.protocol
 
     def tearDown(self):
@@ -156,7 +178,8 @@ class DisconnectionTests(unittest.TestCase):
             it_was_called.yes = True
             return None
         it_was_called.yes = False
-        self.protocol.on_disconnect.addCallback(it_was_called).addErrback(it_was_called)
+        self.protocol.on_disconnect.addCallback(it_was_called)
+        self.protocol.on_disconnect.addErrback(it_was_called)
         f = failure.Failure(error.ConnectionDone("It's all over"))
         self.protocol.connectionLost(f)
         self.assertTrue(it_was_called.yes)
@@ -170,7 +193,8 @@ class DisconnectionTests(unittest.TestCase):
             it_was_called.yes = True
             return None
         it_was_called.yes = False
-        self.protocol.on_disconnect.addCallback(it_was_called).addErrback(it_was_called)
+        self.protocol.on_disconnect.addCallback(it_was_called)
+        self.protocol.on_disconnect.addErrback(it_was_called)
         f = failure.Failure(RuntimeError("The thing didn't do the stuff."))
         self.protocol.connectionLost(f)
         self.assertTrue(it_was_called.yes)
@@ -294,15 +318,21 @@ OK''' % cookietmp.name)
 AUTH METHODS=SAFECOOKIE COOKIEFILE="%s"
 VERSION Tor="0.2.2.35"
 OK''' % cookietmp.name)
-            self.assertTrue('AUTHCHALLENGE SAFECOOKIE ' in self.transport.value())
+            self.assertTrue(
+                'AUTHCHALLENGE SAFECOOKIE ' in self.transport.value()
+            )
             client_nonce = base64.b16decode(self.transport.value().split()[-1])
             self.transport.clear()
             server_nonce = str(bytearray([0] * 32))
-            server_hash = hmac_sha256("Tor safe cookie authentication server-to-controller hash",
-                                      cookiedata + client_nonce + server_nonce)
+            server_hash = hmac_sha256(
+                "Tor safe cookie authentication server-to-controller hash",
+                cookiedata + client_nonce + server_nonce
+            )
 
-            self.send('250 AUTHCHALLENGE SERVERHASH=%s SERVERNONCE=%s' %
-                      (base64.b16encode(server_hash), base64.b16encode(server_nonce)))
+            self.send(
+                '250 AUTHCHALLENGE SERVERHASH=%s SERVERNONCE=%s' %
+                (base64.b16encode(server_hash), base64.b16encode(server_nonce))
+            )
             self.assertTrue('AUTHENTICATE ' in self.transport.value())
 
     def test_authenticate_safecookie_wrong_hash(self):
@@ -310,13 +340,15 @@ OK''' % cookietmp.name)
         server_nonce = str(bytearray([0] * 32))
         server_hash = str(bytearray([0] * 32))
 
-        ## pretend we already did PROTOCOLINFO and read the cookie
-        ## file
+        # pretend we already did PROTOCOLINFO and read the cookie
+        # file
         self.protocol.cookie_data = cookiedata
         self.protocol.client_nonce = server_nonce  # all 0's anyway
         try:
-            self.protocol._safecookie_authchallenge('250 AUTHCHALLENGE SERVERHASH=%s SERVERNONCE=%s' %
-                                                    (base64.b16encode(server_hash), base64.b16encode(server_nonce)))
+            self.protocol._safecookie_authchallenge(
+                '250 AUTHCHALLENGE SERVERHASH=%s SERVERNONCE=%s' %
+                (base64.b16encode(server_hash), base64.b16encode(server_nonce))
+            )
             self.assertTrue(False)
         except RuntimeError, e:
             self.assertTrue('hash not expected' in str(e))
@@ -335,7 +367,10 @@ OK''' % cookietmp.name)
         events = 'GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL'
         self.protocol._bootstrap()
 
-        ## answer all the requests generated by boostrapping etc.
+        # answer all the requests generated by boostrapping etc.
+        self.send("250-signal/names=")
+        self.send("250 OK")
+
         self.send("250-version=foo")
         self.send("250 OK")
 
@@ -346,6 +381,12 @@ OK''' % cookietmp.name)
 
         return d
 
+    def test_bootstrap_tor_does_not_support_signal_names(self):
+        self.protocol._bootstrap()
+        self.send('552 Unrecognized key "signal/names"')
+        valid_signals = ["RELOAD", "DUMP", "DEBUG", "NEWNYM", "CLEARDNSCACHE"]
+        self.assertEqual(self.protocol.valid_signals, valid_signals)
+
     def test_async(self):
         """
         test the example from control-spec.txt to see that we
@@ -362,14 +403,19 @@ OK''' % cookietmp.name)
         return d
 
     def test_async_multiline(self):
-        ## same as above, but i think the 650's can be multline,
-        ## too. Like:
-        ## 650-CIRC 1000 EXTENDED moria1,moria2 0xBEEF
-        ## 650-EXTRAMAGIC=99
-        ## 650 ANONYMITY=high
+        # same as above, but i think the 650's can be multline,
+        # too. Like:
+        # 650-CIRC 1000 EXTENDED moria1,moria2 0xBEEF
+        # 650-EXTRAMAGIC=99
+        # 650 ANONYMITY=high
 
         self.protocol._set_valid_events('CIRC')
-        self.protocol.add_event_listener('CIRC', CallbackChecker("1000 EXTENDED moria1,moria2\nEXTRAMAGIC=99\nANONYMITY=high"))
+        self.protocol.add_event_listener(
+            'CIRC',
+            CallbackChecker(
+                "1000 EXTENDED moria1,moria2\nEXTRAMAGIC=99\nANONYMITY=high"
+            )
+        )
         self.send("250 OK")
 
         d = self.protocol.get_conf("SOCKSPORT ORPORT")
@@ -401,7 +447,10 @@ OK''' % cookietmp.name)
         self.assertEqual(expected, actual)
 
     def test_getinfo_incremental(self):
-        d = self.protocol.get_info_incremental("FOO", functools.partial(self.incremental_check, "bar"))
+        d = self.protocol.get_info_incremental(
+            "FOO",
+            functools.partial(self.incremental_check, "bar")
+        )
         self.send("250+FOO=")
         self.send("bar")
         self.send("bar")
@@ -410,7 +459,10 @@ OK''' % cookietmp.name)
         return d
 
     def test_getinfo_incremental_continuation(self):
-        d = self.protocol.get_info_incremental("FOO", functools.partial(self.incremental_check, "bar"))
+        d = self.protocol.get_info_incremental(
+            "FOO",
+            functools.partial(self.incremental_check, "bar")
+        )
         self.send("250-FOO=")
         self.send("250-bar")
         self.send("250-bar")
@@ -418,7 +470,10 @@ OK''' % cookietmp.name)
         return d
 
     def test_getinfo_one_line(self):
-        d = self.protocol.get_info("foo", functools.partial(self.incremental_check, "bar"))
+        d = self.protocol.get_info(
+            "foo",
+            functools.partial(self.incremental_check, "bar")
+        )
         self.send('250 foo=bar')
         return d
 
@@ -440,16 +495,22 @@ OK''' % cookietmp.name)
         self.assertEqual(v, '')
 
     def test_setconf(self):
-        d = self.protocol.set_conf("foo", "bar").addCallback(functools.partial(self.response_ok))
+        d = self.protocol.set_conf("foo", "bar").addCallback(
+            functools.partial(self.response_ok)
+        )
         self.send("250 OK")
         self._wait(d)
         self.assertEqual(self.transport.value(), "SETCONF foo=bar\r\n")
 
     def test_setconf_with_space(self):
-        d = self.protocol.set_conf("foo", "a value with a space").addCallback(functools.partial(self.response_ok))
+        d = self.protocol.set_conf("foo", "a value with a space")
+        d.addCallback(functools.partial(self.response_ok))
         self.send("250 OK")
         self._wait(d)
-        self.assertEqual(self.transport.value(), 'SETCONF foo="a value with a space"\r\n')
+        self.assertEqual(
+            self.transport.value(),
+            'SETCONF foo="a value with a space"\r\n'
+        )
 
     def test_setconf_multi(self):
         d = self.protocol.set_conf("foo", "bar", "baz", 1)
@@ -457,6 +518,20 @@ OK''' % cookietmp.name)
         self._wait(d)
         self.assertEqual(self.transport.value(), "SETCONF foo=bar baz=1\r\n")
 
+    def test_quit(self):
+        d = self.protocol.quit()
+        self.send("250 OK")
+        self._wait(d)
+        self.assertEqual(self.transport.value(), "QUIT\r\n")
+
+    def test_dot(self):
+        # just checking we don't expode
+        self.protocol.graphviz_data()
+
+    def test_debug(self):
+        self.protocol.start_debug()
+        self.assertTrue(exists('txtorcon-debug.log'))
+
     def error(self, failure):
         print "ERROR", failure
         self.assertTrue(False)
@@ -491,7 +566,10 @@ OK''' % cookietmp.name)
 
     def test_650_after_authenticate(self):
         self.protocol._set_valid_events('CONF_CHANGED')
-        self.protocol.add_event_listener('CONF_CHANGED', CallbackChecker("Foo=bar"))
+        self.protocol.add_event_listener(
+            'CONF_CHANGED',
+            CallbackChecker("Foo=bar")
+        )
         self.send("250 OK")
 
         self.send("650-CONF_CHANGED")
@@ -499,10 +577,13 @@ OK''' % cookietmp.name)
 
     def test_notify_after_getinfo(self):
         self.protocol._set_valid_events('CIRC')
-        self.protocol.add_event_listener('CIRC', CallbackChecker("1000 EXTENDED moria1,moria2"))
+        self.protocol.add_event_listener(
+            'CIRC',
+            CallbackChecker("1000 EXTENDED moria1,moria2")
+        )
         self.send("250 OK")
 
-        d = self.protocol.get_info("FOO")
+        d = self.protocol.get_info("a")
         d.addCallback(CallbackChecker({'a': 'one'})).addErrback(self.fail)
         self.send("250-a=one")
         self.send("250 OK")
@@ -524,16 +605,62 @@ OK''' % cookietmp.name)
         self.assertEqual(self.transport.value(), "GETINFO version\r\n")
         return d
 
+    def test_getinfo_for_descriptor(self):
+        descriptor_info = """250+desc/name/moria1=
+router moria1 128.31.0.34 9101 0 9131
+platform Tor 0.2.5.0-alpha-dev on Linux
+protocols Link 1 2 Circuit 1
+published 2013-07-05 23:48:52
+fingerprint 9695 DFC3 5FFE B861 329B 9F1A B04C 4639 7020 CE31
+uptime 1818933
+bandwidth 512000 62914560 1307929
+extra-info-digest 17D0142F6EBCDF60160EB1794FA6C9717D581F8C
+caches-extra-info
+onion-key
+-----BEGIN RSA PUBLIC KEY-----
+MIGJAoGBALzd4bhz1usB7wpoaAvP+BBOnNIk7mByAKV6zvyQ0p1M09oEmxPMc3qD
+AAm276oJNf0eq6KWC6YprzPWFsXEIdXSqA6RWXCII1JG/jOoy6nt478BkB8TS9I9
+1MJW27ppRaqnLiTmBmM+qzrsgJGwf+onAgUKKH2GxlVgahqz8x6xAgMBAAE=
+-----END RSA PUBLIC KEY-----
+signing-key
+-----BEGIN RSA PUBLIC KEY-----
+MIGJAoGBALtJ9uD7cD7iHjqNA3AgsX9prES5QN+yFQyr2uOkxzhvunnaf6SNhzWW
+bkfylnMrRm/qCz/czcjZO6N6EKHcXmypehvP566B7gAQ9vDsb+l7VZVWgXvzNc2s
+tl3P7qpC08rgyJh1GqmtQTCesIDqkEyWxwToympCt09ZQRq+fIttAgMBAAE=
+-----END RSA PUBLIC KEY-----
+hidden-service-dir
+contact 1024D/28988BF5 arma mit edu
+ntor-onion-key 9ZVjNkf/iLEnD685SpC5kcDytQ7u5ViiI9JOftdbE0k=
+reject *:*
+router-signature
+-----BEGIN SIGNATURE-----
+Y8Tj2e7mPbFJbguulkPEBVYzyO57p4btpWEXvRMD6vxIh/eyn25pehg5dUVBtZlL
+iO3EUE0AEYah2W9gdz8t+i3Dtr0zgqLS841GC/TyDKCm+MKmN8d098qnwK0NGF9q
+01NZPuSqXM1b6hnl2espFzL7XL8XEGRU+aeg+f/ukw4=
+-----END SIGNATURE-----
+.
+250 OK"""
+        d = self.protocol.get_info("desc/name/moria1")
+        d.addCallback(CallbackChecker({'desc/name/moria1': '\n' + '\n'.join(descriptor_info.split('\n')[1:-2])}))
+        d.addErrback(self.fail)
+
+        for line in descriptor_info.split('\n'):
+            self.send(line)
+        return d
+
     def test_addevent(self):
         self.protocol._set_valid_events('FOO BAR')
 
         self.protocol.add_event_listener('FOO', lambda _: None)
-        ## is it dangerous/ill-advised to depend on internal state of
-        ## class under test?
+        # is it dangerous/ill-advised to depend on internal state of
+        # class under test?
         d = self.protocol.defer
         self.send("250 OK")
         self._wait(d)
-        self.assertEqual(self.transport.value().split('\r\n')[-2], "SETEVENTS FOO")
+        self.assertEqual(
+            self.transport.value().split('\r\n')[-2],
+            "SETEVENTS FOO"
+        )
         self.transport.clear()
 
         self.protocol.add_event_listener('BAR', lambda _: None)
@@ -544,7 +671,9 @@ OK''' % cookietmp.name)
         self._wait(d)
 
         try:
-            self.protocol.add_event_listener('SOMETHING_INVALID', lambda _: None)
+            self.protocol.add_event_listener(
+                'SOMETHING_INVALID', lambda _: None
+            )
             self.assertTrue(False)
         except:
             pass
@@ -600,19 +729,19 @@ OK''' % cookietmp.name)
         self.assertEqual(self.transport.value(), 'SETEVENTS STREAM\r\n')
         self.protocol.lineReceived("250 OK")
         self.transport.clear()
-        ## add another one, shouldn't issue a tor command
+        # add another one, shouldn't issue a tor command
         self.protocol.add_event_listener('STREAM', listener1)
         self.assertEqual(self.transport.value(), '')
 
-        ## remove one, should still not issue a tor command
+        # remove one, should still not issue a tor command
         self.protocol.remove_event_listener('STREAM', listener0)
         self.assertEqual(self.transport.value(), '')
 
-        ## remove the other one, NOW should issue a command
+        # remove the other one, NOW should issue a command
         self.protocol.remove_event_listener('STREAM', listener1)
         self.assertEqual(self.transport.value(), 'SETEVENTS \r\n')
 
-        ## try removing invalid event
+        # try removing invalid event
         try:
             self.protocol.remove_event_listener('FOO', listener0)
             self.fail()
@@ -637,8 +766,9 @@ OK''' % cookietmp.name)
 
     def test_newdesc(self):
         """
-        FIXME: this test is now maybe a little silly, it's just testing multiline GETINFO...
-        (Real test is in TorStateTests.test_newdesc_parse)
+        FIXME: this test is now maybe a little silly, it's just testing
+        multiline GETINFO...  (Real test is in
+        TorStateTests.test_newdesc_parse)
         """
 
         self.protocol.get_info_raw('ns/id/624926802351575FF7E4E3D60EFA3BFB56E67E8A')
@@ -665,7 +795,8 @@ p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863
 
     def test_minus_line_no_command(self):
         """
-        haven't seen 600's use - "in the wild" but don't see why it's not possible
+        haven't seen 600's use - "in the wild" but don't see why it's not
+        possible
         """
         self.protocol._set_valid_events('NS')
         self.protocol.add_event_listener('NS', lambda _: None)
@@ -769,7 +900,7 @@ r Unnamed AHe2V2pmj4Yfn0H9+Np3lci7htU T/g7ZLzG/ooqCn+gdLd9Jjh+AEI 2011-12-02 15:
 s Exit Fast Running V2Dir Valid
 w Bandwidth=33
 p reject 25,119,135-139,445,563,1214,4661-4666,6346-6429,6699,6881-6999""")
-        ## the routers list is always keyed with both name and hash
+        # the routers list is always keyed with both name and hash
         self.assertEqual(len(self.controller.routers_by_name), 2)
         self.assertEqual(len(self.controller.routers_by_hash), 2)
         self.assertTrue('right2privassy3' in self.controller.routers)
