@@ -2,14 +2,28 @@ from zope.interface import implements
 from zope.interface.verify import verifyClass
 from twisted.trial import unittest
 from twisted.test import proto_helpers
+from twisted.python.failure import Failure
 from twisted.internet import task, defer
 from twisted.internet.interfaces import IStreamClientEndpoint, IReactorCore
 
 import os
 import tempfile
 
-from txtorcon import TorControlProtocol, TorProtocolError, TorState, Stream, Circuit, build_tor_connection
-from txtorcon.interface import ITorControlProtocol, IStreamAttacher, ICircuitListener, IStreamListener, StreamListenerMixin, CircuitListenerMixin
+from mock import patch
+
+from txtorcon import TorControlProtocol
+from txtorcon import TorProtocolError
+from txtorcon import TorState
+from txtorcon import Stream
+from txtorcon import Circuit
+from txtorcon import build_tor_connection
+from txtorcon import build_local_tor_connection
+from txtorcon.interface import ITorControlProtocol
+from txtorcon.interface import IStreamAttacher
+from txtorcon.interface import ICircuitListener
+from txtorcon.interface import IStreamListener
+from txtorcon.interface import StreamListenerMixin
+from txtorcon.interface import CircuitListenerMixin
 
 
 class CircuitListener(object):
@@ -21,13 +35,21 @@ class CircuitListener(object):
 
     def checker(self, state, circuit, arg=None):
         if self.expected[0][0] != state:
-            raise RuntimeError('Expected event "%s" not "%s".' % (self.expected[0][0], state))
+            raise RuntimeError(
+                'Expected event "%s" not "%s".' %
+                (self.expected[0][0], state)
+            )
         for (k, v) in self.expected[0][1].items():
             if k == 'arg':
                 if v != arg:
-                    raise RuntimeError('Expected argument to have value "%s", not "%s"' % (arg, v))
+                    raise RuntimeError(
+                        'Expected argument to have value "%s", not "%s"' % (arg, v)
+                    )
             elif getattr(circuit, k) != v:
-                raise RuntimeError('Expected attribute "%s" to have value "%s", not "%s"' % (k, v, getattr(circuit, k)))
+                raise RuntimeError(
+                    'Expected attribute "%s" to have value "%s", not "%s"' %
+                    (k, v, getattr(circuit, k))
+                )
         self.expected = self.expected[1:]
 
     def circuit_new(self, circuit):
@@ -58,13 +80,21 @@ class StreamListener(object):
 
     def checker(self, state, stream, arg=None):
         if self.expected[0][0] != state:
-            raise RuntimeError('Expected event "%s" not "%s".' % (self.expected[0][0], state))
+            raise RuntimeError(
+                'Expected event "%s" not "%s".' % (self.expected[0][0], state)
+            )
         for (k, v) in self.expected[0][1].items():
             if k == 'arg':
                 if v != arg:
-                    raise RuntimeError('Expected argument to have value "%s", not "%s"' % (arg, v))
+                    raise RuntimeError(
+                        'Expected argument to have value "%s", not "%s"' %
+                        (arg, v)
+                    )
             elif getattr(stream, k) != v:
-                raise RuntimeError('Expected attribute "%s" to have value "%s", not "%s"' % (k, v, getattr(stream, k)))
+                raise RuntimeError(
+                    'Expected attribute "%s" to have value "%s", not "%s"' %
+                    (k, v, getattr(stream, k))
+                )
         self.expected = self.expected[1:]
 
     def stream_new(self, stream):
@@ -107,7 +137,7 @@ class FakeReactor:
         raise RuntimeError('connectUNIX: ' + str(args))
 
 
-class FakeCircuit:
+class FakeCircuit(Circuit):
 
     def __init__(self, id=-999):
         self.streams = []
@@ -119,7 +149,8 @@ class FakeEndpoint:
     implements(IStreamClientEndpoint)
 
     def get_info_raw(self, keys):
-        return defer.succeed('\r\n'.join(map(lambda k: '%s=' % k, keys.split())))
+        ans = '\r\n'.join(map(lambda k: '%s=' % k, keys.split()))
+        return defer.succeed(ans)
 
     def get_info_incremental(self, key, linecb):
         linecb('%s=' % key)
@@ -130,7 +161,9 @@ class FakeEndpoint:
         self.proto.transport = proto_helpers.StringTransport()
         self.proto.get_info_raw = self.get_info_raw
         self.proto.get_info_incremental = self.get_info_incremental
-        self.proto._set_valid_events('GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL')
+        self.proto._set_valid_events(
+            'GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL'
+        )
 
         return defer.succeed(self.proto)
 
@@ -160,27 +193,13 @@ class FakeEndpointAnswers:
         self.proto.transport = proto_helpers.StringTransport()
         self.proto.get_info_raw = self.get_info_raw
         self.proto.get_info_incremental = self.get_info_incremental
-        self.proto._set_valid_events('GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL')
+        self.proto._set_valid_events(
+            'GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL'
+        )
 
         return defer.succeed(self.proto)
 
 
-class FakeControlProtocol:
-    implements(ITorControlProtocol)  # actually we don't, it's a lie
-
-    def __init__(self):
-        self.is_owned = None
-        self.post_bootstrap = defer.succeed(self)
-        self.on_disconnect = defer.Deferred()
-
-
-class InternalMethodsTests(unittest.TestCase):
-
-    def test_state_diagram(self):
-        TorState(FakeControlProtocol(), bootstrap=False, write_state_diagram=True)
-        self.assertTrue(os.path.exists('routerfsm.dot'))
-
-
 class BootstrapTests(unittest.TestCase):
 
     def confirm_proto(self, x):
@@ -213,13 +232,21 @@ class BootstrapTests(unittest.TestCase):
         return d
 
     def test_build_unix_wrong_permissions(self):
-        self.assertRaises(ValueError, build_tor_connection, (FakeReactor(self), 'a non-existant filename'))
+        self.assertRaises(
+            ValueError,
+            build_tor_connection,
+            (FakeReactor(self), 'a non-existant filename')
+        )
 
     def test_build_wrong_size_tuple(self):
         self.assertRaises(TypeError, build_tor_connection, (1, 2, 3, 4))
 
     def test_build_wrong_args_entirely(self):
-        self.assertRaises(TypeError, build_tor_connection, 'incorrect argument')
+        self.assertRaises(
+            TypeError,
+            build_tor_connection,
+            'incorrect argument'
+        )
 
     def confirm_pid(self, state):
         self.assertEqual(state.tor_pid, 1234)
@@ -256,12 +283,40 @@ class BootstrapTests(unittest.TestCase):
         p.proto.post_bootstrap.callback(p.proto)
         return d
 
+    def test_build_with_answers_guards_unfound_entry(self):
+        p = FakeEndpointAnswers(['',    # ns/all
+                                 '',    # circuit-status
+                                 '',    # stream-status
+                                 '',    # address-mappings/all
+                                 '\n\nkerblam up\nOK\n'     # entry-guards
+                                 ])
+
+        d = build_tor_connection(p, build_state=True)
+        d.addCallback(self.confirm_state)
+        d.addCallback(self.confirm_no_pid)
+        p.proto.post_bootstrap.callback(p.proto)
+        return d
+
+    def test_build_local_unix(self):
+        reactor = FakeReactor(self)
+        d = build_local_tor_connection(reactor)
+        d.addErrback(lambda _: None)
+        return d
+
+    def test_build_local_tcp(self):
+        reactor = FakeReactor(self)
+        d = build_local_tor_connection(reactor, socket=None)
+        d.addErrback(lambda _: None)
+        return d
+
 
 class StateTests(unittest.TestCase):
 
     def setUp(self):
         self.protocol = TorControlProtocol()
         self.state = TorState(self.protocol)
+        # avoid spew in trial logs; state prints this by default
+        self.state._attacher_error = lambda f: f
         self.protocol.connectionMade = lambda: None
         self.transport = proto_helpers.StringTransport()
         self.protocol.makeConnection(self.transport)
@@ -281,73 +336,66 @@ class StateTests(unittest.TestCase):
         self.state.set_attacher(attacher, FakeReactor(self))
         self.state._stream_update("76 CLOSED 0 www.example.com:0 REASON=DONE")
 
+    def test_attacher_error_handler(self):
+        # make sure error-handling "does something" that isn't blowing up
+        with patch('sys.stdout') as fake_stdout:
+            TorState(self.protocol)._attacher_error(Failure(RuntimeError("quote")))
+
     def test_stream_update(self):
-        ## we use a circuit ID of 0 so it doesn't try to look anything up but it's
-        ## not really correct to have a  SUCCEEDED w/o a valid circuit, I don't think
+        # we use a circuit ID of 0 so it doesn't try to look anything
+        # up but it's not really correct to have a SUCCEEDED w/o a
+        # valid circuit, I don't think
         self.state._stream_update('1610 SUCCEEDED 0 74.125.224.243:80')
         self.assertTrue(1610 in self.state.streams)
 
     def test_single_streams(self):
         self.state.circuits[496] = FakeCircuit(496)
-        self.state._stream_status('stream-status=123 SUCCEEDED 496 www.example.com:6667')
+        self.state._stream_status(
+            'stream-status=123 SUCCEEDED 496 www.example.com:6667'
+        )
         self.assertEqual(len(self.state.streams), 1)
 
+    def test_multiple_streams(self):
+        self.state.circuits[496] = FakeCircuit(496)
+        self.state._stream_status(
+            '\r\n'.join([
+                'stream-status=',
+                '123 SUCCEEDED 496 www.example.com:6667',
+                '124 SUCCEEDED 496 www.example.com:6667',
+            ])
+        )
+        self.assertEqual(len(self.state.streams), 2)
+
     def send(self, line):
         self.protocol.dataReceived(line.strip() + "\r\n")
 
+    @defer.inlineCallbacks
     def test_bootstrap_callback(self):
         '''
         FIXME: something is still screwy with this; try throwing an
         exception from TorState.bootstrap and we'll just hang...
         '''
 
-        d = self.state.post_bootstrap
+        from test_torconfig import FakeControlProtocol
+        protocol = FakeControlProtocol(
+            [
+                "ns/all=",  # ns/all
+                "",  # circuit-status
+                "",  # stream-status
+                "",  # address-mappings/all
+                "entry-guards=\r\n$0000000000000000000000000000000000000000=name up\r\n$1111111111111111111111111111111111111111=foo up\r\n$9999999999999999999999999999999999999999=eman unusable 2012-01-01 22:00:00\r\n",  # entry-guards
+                "99999",  # process/pid
+                "??",  # ip-to-country/0.0.0.0
+            ]
+        )
 
-        self.protocol._set_valid_events(' '.join(self.state.event_map.keys()))
-        self.state._bootstrap()
+        state = yield TorState.from_protocol(protocol)
 
-        self.send("250+ns/all=")
-        self.send(".")
-        self.send("250 OK")
-
-        self.send("250+circuit-status=")
-        self.send(".")
-        self.send("250 OK")
-
-        self.send("250-stream-status=")
-        self.send("250 OK")
-
-        self.send("250-address-mappings/all=")
-        self.send("250 OK")
-
-        for ignored in self.state.event_map.items():
-            self.send("250 OK")
-
-        fakerouter = object()
-        self.state.routers['$0000000000000000000000000000000000000000'] = fakerouter
-        self.state.routers['$9999999999999999999999999999999999999999'] = fakerouter
-        self.send("250+entry-guards=")
-        self.send("$0000000000000000000000000000000000000000=name up")
-        self.send("$1111111111111111111111111111111111111111=foo up")
-        self.send("$9999999999999999999999999999999999999999=eman unusable 2012-01-01 22:00:00")
-        self.send(".")
-        self.send("250 OK")
-
-        ## implicitly created Router object for the $1111...11 lookup
-        ## but 0.0.0.0 will have to country, so Router will ask Tor
-        ## for one via GETINFO ip-to-country
-        self.send("250-ip-to-country/0.0.0.0=??")
-        self.send("250 OK")
-
-        self.assertEqual(len(self.state.entry_guards), 2)
-        self.assertTrue('$0000000000000000000000000000000000000000' in self.state.entry_guards)
-        self.assertEqual(self.state.entry_guards['$0000000000000000000000000000000000000000'], fakerouter)
-        self.assertTrue('$1111111111111111111111111111111111111111' in self.state.entry_guards)
-
-        self.assertEqual(len(self.state.unusable_entry_guards), 1)
-        self.assertTrue('$9999999999999999999999999999999999999999' in self.state.unusable_entry_guards[0])
-
-        return d
+        self.assertEqual(len(state.entry_guards), 2)
+        self.assertTrue('$0000000000000000000000000000000000000000' in state.entry_guards)
+        self.assertTrue('$1111111111111111111111111111111111111111' in state.entry_guards)
+        self.assertEqual(len(state.unusable_entry_guards), 1)
+        self.assertTrue('$9999999999999999999999999999999999999999' in state.unusable_entry_guards[0])
 
     def test_bootstrap_existing_addresses(self):
         '''
@@ -448,7 +496,11 @@ class StateTests(unittest.TestCase):
         self.send("250 OK")
         self.state.set_attacher(None, fr)
         self.send("250 OK")
-        self.assertEqual(self.transport.value(), 'SETCONF __LeaveStreamsUnattached=1\r\nSETCONF __LeaveStreamsUnattached=0\r\n')
+        self.assertEqual(
+            self.transport.value(),
+            'SETCONF __LeaveStreamsUnattached=1\r\nSETCONF'
+            ' __LeaveStreamsUnattached=0\r\n'
+        )
 
     def test_attacher(self):
         class MyAttacher(object):
@@ -513,11 +565,12 @@ class StateTests(unittest.TestCase):
                 return defer.succeed(self.answer)
 
         self.state.circuits[1] = FakeCircuit(1)
+        self.state.circuits[1].state = 'BUILT'
         attacher = MyAttacher(self.state.circuits[1])
         self.state.set_attacher(attacher, FakeReactor(self))
 
-        ## boilerplate to finish enough set-up in the protocol so it
-        ## works
+        # boilerplate to finish enough set-up in the protocol so it
+        # works
         events = 'GUARD STREAM CIRC NS NEWCONSENSUS ORCONN NEWDESC ADDRMAP STATUS_GENERAL'
         self.protocol._set_valid_events(events)
         self.state._add_events()
@@ -531,6 +584,7 @@ class StateTests(unittest.TestCase):
         self.assertEqual(len(self.protocol.commands), 1)
         self.assertEqual(self.protocol.commands[0][1], 'ATTACHSTREAM 1 1')
 
+    @defer.inlineCallbacks
     def test_attacher_errors(self):
         class MyAttacher(object):
             implements(IStreamAttacher)
@@ -550,7 +604,7 @@ class StateTests(unittest.TestCase):
         stream.id = 3
         msg = ''
         try:
-            self.state._maybe_attach(stream)
+            yield self.state._maybe_attach(stream)
         except Exception, e:
             msg = str(e)
         self.assertTrue('circuit unknown' in msg)
@@ -558,11 +612,19 @@ class StateTests(unittest.TestCase):
         attacher.answer = self.state.circuits[1]
         msg = ''
         try:
-            self.state._maybe_attach(stream)
+            yield self.state._maybe_attach(stream)
         except Exception, e:
             msg = str(e)
         self.assertTrue('only attach to BUILT' in msg)
 
+        attacher.answer = 'not a Circuit instance'
+        msg = ''
+        try:
+            yield self.state._maybe_attach(stream)
+        except Exception, e:
+            msg = str(e)
+        self.assertTrue('Circuit instance' in msg)
+
     def test_attacher_no_attach(self):
         class MyAttacher(object):
             implements(IStreamAttacher)
@@ -610,7 +672,12 @@ class StateTests(unittest.TestCase):
         stream = Stream(self.state)
         stream.id = 1
         self.state.streams[1] = stream
-        self.assertRaises(ValueError, self.state.close_stream, stream, 'FOO_INVALID_REASON')
+        self.assertRaises(
+            ValueError,
+            self.state.close_stream,
+            stream,
+            'FOO_INVALID_REASON'
+        )
 
     def test_close_circuit_with_id(self):
         circuit = Circuit(self.state)
@@ -662,7 +729,7 @@ class StateTests(unittest.TestCase):
         for ignored in self.state.event_map.items():
             self.send("250 OK")
 
-        ## we use this router later on in an EXTEND
+        # we use this router later on in an EXTEND
         self.state._update_network_status("""ns/all=
 r PPrivCom012 2CGDscCeHXeV/y1xFrq1EGqj5g4 QX7NVLwx7pwCuk6s8sxB4rdaCKI 2011-12-20 08:34:19 84.19.178.6 9001 0
 s Fast Guard Running Stable Unnamed Valid
@@ -674,22 +741,22 @@ p reject 1-65535""")
                     ('extend', {'id': 123})
                     ]
         listen = CircuitListener(expected)
-        ## first add a Circuit before we listen
+        # first add a Circuit before we listen
         self.protocol.dataReceived("650 CIRC 123 LAUNCHED PURPOSE=GENERAL\r\n")
         self.assertEqual(len(self.state.circuits), 1)
 
-        ## make sure we get added to existing circuits
+        # make sure we get added to existing circuits
         self.state.add_circuit_listener(listen)
         self.assertTrue(listen in self.state.circuits.values()[0].listeners)
 
-        ## now add a Circuit after we started listening
+        # now add a Circuit after we started listening
         self.protocol.dataReceived("650 CIRC 456 LAUNCHED PURPOSE=GENERAL\r\n")
         self.assertEqual(len(self.state.circuits), 2)
         self.assertTrue(listen in self.state.circuits.values()[0].listeners)
         self.assertTrue(listen in self.state.circuits.values()[1].listeners)
 
-        ## now update the first Circuit to ensure we're really, really
-        ## listening
+        # now update the first Circuit to ensure we're really, really
+        # listening
         self.protocol.dataReceived("650 CIRC 123 EXTENDED $D82183B1C09E1D7795FF2D7116BAB5106AA3E60E~PPrivCom012 PURPOSE=GENERAL\r\n")
         self.assertEqual(len(listen.expected), 0)
 
@@ -797,7 +864,7 @@ p accept 43,53
         self.assertEqual(r.bandwidth, 518000)
         self.assertEqual(len(self.state.routers_by_name['fake']), 2)
 
-        ## now we do an update
+        # now we do an update
         self.state._update_network_status('''ns/all=
 r fake YkkmgCNRV1/35OPWDvo7+1bmfoo tanLV/4ZfzpYQW0xtGFqAa46foo 2011-12-12 16:29:16 12.45.56.78 443 80
 s Exit Fast Guard HSDir Named Running Stable V2Dir Valid FutureProof Authority
@@ -818,8 +885,8 @@ p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863
         errors.
         """
 
-        ## bootstrap the TorState so we can send it a "real" 650
-        ## update
+        # bootstrap the TorState so we can send it a "real" 650
+        # update
 
         self.protocol._set_valid_events(' '.join(self.state.event_map.keys()))
         self.state._bootstrap()
@@ -846,7 +913,7 @@ p accept 43,53,79-81,110,143,194,220,443,953,989-990,993,995,1194,1293,1723,1863
 
         self.send("250 OK")
 
-        ## state is now bootstrapped, we can send our NEWCONSENSUS update
+        # state is now bootstrapped, we can send our NEWCONSENSUS update
 
         self.protocol.dataReceived('\r\n'.join('''650+NEWCONSENSUS
 r Unnamed ABJlguUFz1lvQS0jq8nhTdRiXEk /zIVUg1tKMUeyUBoyimzorbQN9E 2012-05-23 01:10:22 219.94.255.254 9001 0
@@ -875,8 +942,8 @@ p reject 1-65535
         errors.
         """
 
-        ## bootstrap the TorState so we can send it a "real" 650
-        ## update
+        # bootstrap the TorState so we can send it a "real" 650
+        # update
 
         self.protocol._set_valid_events(' '.join(self.state.event_map.keys()))
         self.state._bootstrap()
@@ -903,7 +970,7 @@ p reject 1-65535
 
         self.send("250 OK")
 
-        ## state is now bootstrapped, we can send our NEWCONSENSUS update
+        # state is now bootstrapped, we can send our NEWCONSENSUS update
 
         self.protocol.dataReceived('\r\n'.join('''650+NEWCONSENSUS
 r Unnamed ABJlguUFz1lvQS0jq8nhTdRiXEk /zIVUg1tKMUeyUBoyimzorbQN9E 2012-05-23 01:10:22 219.94.255.254 9001 0
@@ -922,8 +989,8 @@ w Bandwidth=166
         errors.
         """
 
-        ## bootstrap the TorState so we can send it a "real" 650
-        ## update
+        # bootstrap the TorState so we can send it a "real" 650
+        # update
 
         self.protocol._set_valid_events(' '.join(self.state.event_map.keys()))
         self.state._bootstrap()
@@ -950,7 +1017,7 @@ w Bandwidth=166
 
         self.send("250 OK")
 
-        ## state is now bootstrapped, we can send our NEWCONSENSUS update
+        # state is now bootstrapped, we can send our NEWCONSENSUS update
 
         self.protocol.dataReceived('\r\n'.join('''650+NEWCONSENSUS
 r Unnamed ABJlguUFz1lvQS0jq8nhTdRiXEk /zIVUg1tKMUeyUBoyimzorbQN9E 2012-05-23 01:10:22 219.94.255.254 9001 0
@@ -1014,20 +1081,24 @@ s Fast Guard Running Stable Valid
         path = []
         for x in range(3):
             path.append(FakeRouter("$%040d" % x))
-        ## can't just check flags for guard status, need to know if
-        ## it's in the running Tor's notion of Entry Guards
+        # can't just check flags for guard status, need to know if
+        # it's in the running Tor's notion of Entry Guards
         path[0].flags = ['guard']
 
-        self.state.build_circuit(path)
+        self.state.build_circuit(path, using_guards=True)
         self.assertEqual(self.transport.value(), 'EXTENDCIRCUIT 0 0000000000000000000000000000000000000000,0000000000000000000000000000000000000001,0000000000000000000000000000000000000002\r\n')
-        ## should have gotten a warning about this not being an entry
-        ## guard
+        # should have gotten a warning about this not being an entry
+        # guard
         self.assertEqual(len(self.flushWarnings()), 1)
 
     def test_build_circuit_no_routers(self):
         self.state.build_circuit()
         self.assertEqual(self.transport.value(), 'EXTENDCIRCUIT 0\r\n')
 
+    def test_build_circuit_unfound_router(self):
+        self.state.build_circuit(routers=['AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'], using_guards=False)
+        self.assertEqual(self.transport.value(), 'EXTENDCIRCUIT 0 AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\r\n')
+
     def circuit_callback(self, circ):
         self.assertTrue(isinstance(circ, Circuit))
         self.assertEqual(circ.id, 1234)
@@ -1041,19 +1112,19 @@ s Fast Guard Running Stable Valid
         path = []
         for x in range(3):
             path.append(FakeRouter("$%040d" % x))
-        ## can't just check flags for guard status, need to know if
-        ## it's in the running Tor's notion of Entry Guards
+        # can't just check flags for guard status, need to know if
+        # it's in the running Tor's notion of Entry Guards
         path[0].flags = ['guard']
 
-        ## FIXME TODO we should verify we get a circuit_new event for
-        ## this circuit
+        # FIXME TODO we should verify we get a circuit_new event for
+        # this circuit
 
-        d = self.state.build_circuit(path)
+        d = self.state.build_circuit(path, using_guards=True)
         d.addCallback(self.circuit_callback)
         self.assertEqual(self.transport.value(), 'EXTENDCIRCUIT 0 0000000000000000000000000000000000000000,0000000000000000000000000000000000000001,0000000000000000000000000000000000000002\r\n')
         self.send('250 EXTENDED 1234')
-        ## should have gotten a warning about this not being an entry
-        ## guard
+        # should have gotten a warning about this not being an entry
+        # guard
         self.assertEqual(len(self.flushWarnings()), 1)
         return d
 
