@@ -1,10 +1,16 @@
+from mock import patch
 from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.internet.endpoints import TCP4ServerEndpoint
 from twisted.internet.interfaces import IProtocolFactory
 from zope.interface import implements
 
-from txtorcon.util import process_from_address, delete_file_or_tree, find_keywords, ip_from_int, find_tor_binary
+from txtorcon.util import process_from_address
+from txtorcon.util import delete_file_or_tree
+from txtorcon.util import find_keywords
+from txtorcon.util import ip_from_int
+from txtorcon.util import find_tor_binary
+from txtorcon.util import maybe_ip_addr
 
 import os
 import tempfile
@@ -41,13 +47,33 @@ class TestGeoIpDatabaseLoading(unittest.TestCase):
         from txtorcon import util
         self.assertRaises(IOError, util.create_geoip, '_missing_path_')
 
+    def test_missing_geoip_module(self):
+        "return none if geoip module is missing"
+        from txtorcon import util
+        _GeoIP = util.GeoIP
+        util.GeoIP = None
+        (fd, f) = tempfile.mkstemp()
+        ret_val = util.create_geoip(f)
+        delete_file_or_tree(f)
+        util.GeoIP = _GeoIP
+        self.assertEquals(ret_val, None)
+
+    def test_return_geoip_object(self):
+        from txtorcon import util
+        (fd, f) = tempfile.mkstemp()
+        ret_val = util.create_geoip(f)
+        delete_file_or_tree(f)
+        self.assertEquals(type(ret_val).__name__, 'GeoIP')
+
 
 class TestFindKeywords(unittest.TestCase):
 
     def test_filter(self):
         "make sure we filter out keys that look like router IDs"
-        self.assertEqual(find_keywords("foo=bar $1234567890=routername baz=quux".split()),
-                         {'foo': 'bar', 'baz': 'quux'})
+        self.assertEqual(
+            find_keywords("foo=bar $1234567890=routername baz=quux".split()),
+            {'foo': 'bar', 'baz': 'quux'}
+        )
 
 
 class FakeGeoIP(object):
@@ -175,13 +201,14 @@ class TestProcessFromUtil(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_real_addr(self):
-        ## FIXME should choose a port which definitely isn't used.
+        # FIXME should choose a port which definitely isn't used.
 
-        ## it's apparently frowned upon to use the "real" reactor in
-        ## tests, but I was using "nc" before, and I think this is
-        ## preferable.
+        # it's apparently frowned upon to use the "real" reactor in
+        # tests, but I was using "nc" before, and I think this is
+        # preferable.
         from twisted.internet import reactor
-        listener = yield TCP4ServerEndpoint(reactor, 9887).listen(FakeProtocolFactory())
+        ep = TCP4ServerEndpoint(reactor, 9887)
+        listener = yield ep.listen(FakeProtocolFactory())
 
         try:
             pid = process_from_address('0.0.0.0', 9887, self.fakestate)
@@ -220,7 +247,7 @@ class TestDelete(unittest.TestCase):
 class TestFindTor(unittest.TestCase):
 
     def test_simple_find_tor(self):
-        ## just test that this doesn't raise an exception
+        # just test that this doesn't raise an exception
         find_tor_binary()
 
     def test_find_tor_globs(self):
@@ -230,3 +257,23 @@ class TestFindTor(unittest.TestCase):
     def test_find_tor_unfound(self):
         "test searching by globs"
         self.assertEqual(None, find_tor_binary(system_tor=False, globs=()))
+
+    @patch('txtorcon.util.subprocess.Popen')
+    def test_find_ioerror(self, popen):
+        "test searching with which, but it fails"
+        popen.side_effect = OSError
+        self.assertEqual(None, find_tor_binary(system_tor=True, globs=()))
+
+
+class TestIpAddr(unittest.TestCase):
+
+    @patch('txtorcon.util.ipaddr')
+    def test_create_ipaddr(self, ipaddr):
+        ip = maybe_ip_addr('1.2.3.4')
+
+    @patch('txtorcon.util.ipaddr')
+    def test_create_ipaddr(self, ipaddr):
+        def foo(blam):
+            raise ValueError('testing')
+        ipaddr.IPAddress.side_effect = foo
+        ip = maybe_ip_addr('1.2.3.4')
