@@ -1,3 +1,10 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+from __future__ import with_statement
+
 from txtorcon.interface import IAddrListener
 from txtorcon.util import maybe_ip_addr
 
@@ -42,7 +49,7 @@ class Addr(object):
         if gmtexpires is None:
             if len(args) == 3:
                 gmtexpires = expires
-            elif len(args) > 3:
+            else:
                 if args[2] == 'NEVER':
                     gmtexpires = args[2]
                 else:
@@ -57,13 +64,13 @@ class Addr(object):
 
         fmt = "%Y-%m-%d %H:%M:%S"
 
-        ## if we already have expiry times, etc then we want to
-        ## properly delay our timeout
+        # if we already have expiry times, etc then we want to
+        # properly delay our timeout
 
         oldexpires = self.expires
 
         if gmtexpires.upper() == 'NEVER':
-            ## FIXME can I just select a date 100 years in the future instead?
+            # FIXME can I just select a date 100 years in the future instead?
             self.expires = None
         else:
             self.expires = datetime.datetime.strptime(gmtexpires, fmt)
