@@ -1,13 +1,20 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+from __future__ import with_statement
+
 import time
 import datetime
 
 from twisted.python import log
 from twisted.internet import defer
-from interface import IRouterContainer
+from .interface import IRouterContainer
 
 from txtorcon.util import find_keywords
 
-#look like "2014-01-25T02:12:14.593772"
+# look like "2014-01-25T02:12:14.593772"
 TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
 
@@ -54,7 +61,7 @@ class Circuit(object):
           - TESTING
           - CONTROLLER
 
-        For most purposes, you'll want to look at GENERAL circuits only.
+    For most purposes, you'll want to look at GENERAL circuits only.
 
 
     :ivar id:
@@ -77,19 +84,43 @@ class Circuit(object):
         self.build_flags = []
         self.flags = {}
 
-        ## this is used to hold a Deferred that will callback() when
-        ## this circuit is being CLOSED or FAILED.
+        # this is used to hold a Deferred that will callback() when
+        # this circuit is being CLOSED or FAILED.
         self._closing_deferred = None
 
-        ## caches parsed value for time_created()
+        # caches parsed value for time_created()
         self._time_created = None
 
+        # all notifications for when_built
+        self._when_built = []
+
+    # XXX backwards-compat for old .is_built for now
+    @property
+    def is_built(self):
+        return self.when_built()
+
+    def when_built(self):
+        """
+        Returns a Deferred that is callback()'d (with this Circuit
+        instance) when this circuit hits BUILT.
+
+        If it's already BUILT when this is called, you get an
+        already-successful Deferred; otherwise, the state must change
+        to BUILT.
+        """
+        d = defer.Deferred()
+        if self.state == 'BUILT':
+            d.callback(self)
+        else:
+            self._when_built.append(d)
+        return d
+
     @property
     def time_created(self):
         if self._time_created is not None:
             return self._time_created
         if 'TIME_CREATED' in self.flags:
-            ## strip off milliseconds
+            # strip off milliseconds
             t = self.flags['TIME_CREATED'].split('.')[0]
             tstruct = time.strptime(t, TIME_FORMAT)
             self._time_created = datetime.datetime(*tstruct[:7])
@@ -105,7 +136,7 @@ class Circuit(object):
     def close(self, **kw):
         """
         This asks Tor to close the underlying circuit object. See
-        :method:`txtorcon.torstate.TorState.close_circuit`
+        :meth:`txtorcon.torstate.TorState.close_circuit`
         for details.
 
         You may pass keyword arguments to take care of any Flags Tor
@@ -150,7 +181,7 @@ class Circuit(object):
         return flags
 
     def update(self, args):
-        ##print "Circuit.update:",args
+        # print "Circuit.update:",args
         if self.id is None:
             self.id = int(args[0])
             [x.circuit_new(self) for x in self.listeners]
@@ -177,12 +208,15 @@ class Circuit(object):
 
         if self.state == 'BUILT':
             [x.circuit_built(self) for x in self.listeners]
+            for d in self._when_built:
+                d.callback(self)
+            self._when_built = []
 
         elif self.state == 'CLOSED':
             if len(self.streams) > 0:
-                ## FIXME it seems this can/does happen if a remote
-                ## router crashes or otherwise shuts down a circuit
-                ## with streams on it still
+                # FIXME it seems this can/does happen if a remote
+                # router crashes or otherwise shuts down a circuit
+                # with streams on it still
                 log.err(RuntimeError("Circuit is %s but still has %d streams" %
                                      (self.state, len(self.streams))))
             flags = self._create_flags(kw)
@@ -228,8 +262,8 @@ class Circuit(object):
             if p[0] != '$':
                 break
 
-            ## this will create a Router if we give it a router
-            ## LongName that doesn't yet exist
+            # this will create a Router if we give it a router
+            # LongName that doesn't yet exist
             router = self.router_container.router_from_id(p)
 
             self.path.append(router)
@@ -238,6 +272,6 @@ class Circuit(object):
                 oldpath = self.path
 
     def __str__(self):
-        path = ' '.join(map(lambda x: x.ip, self.path))
+        path = ' '.join([x.ip for x in self.path])
         return "<Circuit %d %s [%s] for %s>" % (self.id, self.state, path,
                                                 self.purpose)
