@@ -1,3 +1,9 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import with_statement
+
 import os
 import shutil
 import weakref
@@ -6,23 +12,37 @@ import functools
 
 from txtorcon.util import available_tcp_port
 
-from twisted.internet import defer
+# backwards-compatibility dance: we "should" be using the
+# ...WithReactor class, but in Twisted prior to 14, there is no such
+# class (and the parse() doesn't provide a 'reactor' argument).
+try:
+    from twisted.internet.interfaces import IStreamClientEndpointStringParserWithReactor
+    _HAVE_TX_14 = True
+except ImportError:
+    from twisted.internet.interfaces import IStreamClientEndpointStringParser as IStreamClientEndpointStringParserWithReactor
+    _HAVE_TX_14 = False
+
+from twisted.internet import defer, reactor
 from twisted.python import log
 from twisted.internet.interfaces import IStreamServerEndpointStringParser
 from twisted.internet.interfaces import IStreamServerEndpoint
+from twisted.internet.interfaces import IStreamClientEndpoint
 from twisted.internet.interfaces import IListeningPort
 from twisted.internet.interfaces import IAddress
-from twisted.internet.interfaces import IReactorCore
 from twisted.internet.endpoints import serverFromString
 from twisted.internet.endpoints import clientFromString
+from twisted.internet.endpoints import TCP4ClientEndpoint
+from twisted.internet import error
 from twisted.plugin import IPlugin
 from twisted.python.util import FancyEqMixin
 
 from zope.interface import implementer
 from zope.interface import Interface, Attribute
 
-from torconfig import TorConfig, launch_tor, HiddenService
-from torstate import build_tor_connection
+from txsocksx.client import SOCKS5ClientEndpoint
+
+from .torconfig import TorConfig, launch_tor, HiddenService
+from .torstate import build_tor_connection
 
 
 _global_tor_config = None
@@ -34,9 +54,11 @@ _global_tor_lock = defer.DeferredLock()
 @defer.inlineCallbacks
 def get_global_tor(reactor, control_port=None,
                    progress_updates=None,
-                   _tor_launcher=lambda r, c, p: launch_tor(c, r, progress_updates=p)):
+                   _tor_launcher=lambda r, c, p: launch_tor(
+                       c, r, progress_updates=p)):
     """
-    See description of :class:`txtorcon.TCPHiddenServiceEndpoint`'s class-method ``global_tor``
+    See description of :class:`txtorcon.TCPHiddenServiceEndpoint`'s
+    class-method ``global_tor``
 
     :param control_port:
         a TCP port upon which to run the launched Tor's
@@ -47,7 +69,8 @@ def get_global_tor(reactor, control_port=None,
         is called when Tor announcing some progress setting itself up.
 
     :returns:
-        a ``Deferred`` that fires a :class:`txtorcon.TorConfig` which is bootstrapped.
+        a ``Deferred`` that fires a :class:`txtorcon.TorConfig` which is
+        bootstrapped.
 
     The _tor_launcher keyword arg is internal-only.
     """
@@ -66,7 +89,8 @@ def get_global_tor(reactor, control_port=None,
         else:
             cp = _global_tor_config.ControlPort
             if control_port is not None and control_port != cp:
-                raise RuntimeError("ControlPort is %s, you wanted %s" % (cp, control_port))
+                raise RuntimeError(
+                    "ControlPort is %s, you wanted %s" % (cp, control_port))
 
         defer.returnValue(_global_tor_config)
     finally:
@@ -100,8 +124,7 @@ class IProgressProvider(Interface):
 
 @implementer(IStreamServerEndpoint, IProgressProvider)
 class TCPHiddenServiceEndpoint(object):
-    """
-    This represents something listening on an arbitrary local port
+    """This represents something listening on an arbitrary local port
     that has a Tor configured with a Hidden Service pointing at
     it. :api:`twisted.internet.endpoints.TCP4ServerEndpoint
     <TCP4ServerEndpoint>` is used under the hood to do the local
@@ -115,7 +138,8 @@ class TCPHiddenServiceEndpoint(object):
 
     1. system_tor(...) connects to an already-started tor on the
        endpoint you specify; stricly speaking not a "system" tor since
-       you could have spawned it some other way. See `Tor bug 11291 <https://trac.torproject.org/projects/tor/ticket/11291>`_
+       you could have spawned it some other way. See `Tor bug 11291
+       <https://trac.torproject.org/projects/tor/ticket/11291>`_
        however.
 
     2. global_tor(...) refers to a single possible Tor instance
@@ -133,10 +157,10 @@ class TCPHiddenServiceEndpoint(object):
 
     No matter how you came by your instance, calling `listen()` on it
     causes Tor to be launched or connected-to, your hidden service to
-    be added, (XXX and check the descriptor is uploaded! FIXME) and
-    you get a ``Deferred`` with an ``IListeningPort`` whose
-    ``getHost()`` will return a :class:`txtorcon.TorOnionAddress`. The port
-    object will also implement :class:`txtorcon.IHiddenService` so you can get the
+    be added, checks that the descriptor is uploaded and you get a
+    ``Deferred`` with an ``IListeningPort`` whose ``getHost()`` will
+    return a :class:`txtorcon.TorOnionAddress`. The port object will
+    also implement :class:`txtorcon.IHiddenService` so you can get the
     locally-listening address and hidden serivce directory::
 
         endpoint = ...
@@ -157,6 +181,7 @@ class TCPHiddenServiceEndpoint(object):
 
     :ivar hiddenServiceDir: the data directory, either passed in or created
         with ``tempfile.mkstemp``
+
     """
 
     @classmethod
@@ -165,7 +190,6 @@ class TCPHiddenServiceEndpoint(object):
         """
         This returns a TCPHiddenServiceEndpoint connected to the
         endpoint you specify in `control_endpoint`. After connecting, a
-
         single hidden service is added. The endpoint can be a Unix
         socket if Tor's `ControlSocket` option was used (instead of
         `ControlPort`).
@@ -173,7 +197,7 @@ class TCPHiddenServiceEndpoint(object):
         .. note::
 
             If Tor bug #11291 is not yet fixed, this won't work if you
-            only have Group access.
+            only have Group access. XXX FIXME re-test
         """
 
         @defer.inlineCallbacks
@@ -329,8 +353,8 @@ class TCPHiddenServiceEndpoint(object):
 
     @defer.inlineCallbacks
     def listen(self, protocolfactory):
-        """
-        Implement :api:`twisted.internet.interfaces.IStreamServerEndpoint <IStreamServerEndpoint>`.
+        """Implement :api:`twisted.internet.interfaces.IStreamServerEndpoint
+        <IStreamServerEndpoint>`.
 
         Returns a Deferred that delivers an
         :api:`twisted.internet.interfaces.IListeningPort` implementation.
@@ -352,6 +376,11 @@ class TCPHiddenServiceEndpoint(object):
         FIXME TODO: also listen for an INFO-level Tor message (does
         exist, #tor-dev says) that indicates the hidden service's
         descriptor is published.
+
+        It is "connection_dir_client_reached_eof(): Uploaded
+        rendezvous descriptor (status 200 ("Service descriptor (v2)
+        stored"))" at INFO level.
+
         """
 
         self.protocolfactory = protocolfactory
@@ -373,16 +402,43 @@ class TCPHiddenServiceEndpoint(object):
 
         # NOTE at some point, we can support unix sockets here
         # once Tor does. See bug #XXX
+
+        # specifically NOT creating the hidden-service dir; letting
+        # Tor do it will more-likely result in a usable situation...
         if not os.path.exists(self.hidden_service_dir):
-                log.msg('Creating "%s".' % self.hidden_service_dir)
-                os.makedirs(self.hidden_service_dir)
-        self.hiddenservice = HiddenService(self.config, self.hidden_service_dir,
-                                           ['%d 127.0.0.1:%d' % (self.public_port,
-                                                                 self.local_port)])
-        self.config.HiddenServices.append(self.hiddenservice)
+            log.msg(
+                'Noting that "%s" does not exist; letting Tor create it.' %
+                self.hidden_service_dir
+            )
+
+        # listen for the descriptor upload event
+        info_callback = defer.Deferred()
+
+        def info_event(msg):
+            # XXX giant hack here; Right Thing would be to implement a
+            # "real" event in Tor and listen for that.
+            if 'Service descriptor (v2) stored' in msg:
+                info_callback.callback(None)
+        self.config.protocol.add_event_listener('INFO', info_event)
+
+        if self.hidden_service_dir not in [hs.dir for hs in self.config.HiddenServices]:
+            self.hiddenservice = HiddenService(
+                self.config, self.hidden_service_dir,
+                ['%d 127.0.0.1:%d' % (self.public_port, self.local_port)],
+                group_readable=1)
+            self.config.HiddenServices.append(self.hiddenservice)
         yield self.config.save()
 
-        log.msg('Started hidden service "%s" on port %d' % (self.onion_uri, self.public_port))
+        self._tor_progress_update(100.0, 'wait_descriptor',
+                                  'Waiting for descriptor upload...')
+        yield info_callback  # awaits an INFO log-line from Tor .. sketchy
+        yield self.config.protocol.remove_event_listener('INFO', info_event)
+        self._tor_progress_update(100.0, 'wait_descriptor',
+                                  'At least one descriptor uploaded.')
+
+        log.msg(
+            'Started hidden service "%s" on port %d' %
+            (self.onion_uri, self.public_port))
         log.msg('Keys are in "%s".' % (self.hidden_service_dir,))
         defer.returnValue(TorOnionListeningPort(self.tcp_listening_port,
                                                 self.hidden_service_dir,
@@ -394,7 +450,8 @@ class TCPHiddenServiceEndpoint(object):
 @implementer(IAddress)
 class TorOnionAddress(FancyEqMixin, object):
     """
-    A ``TorOnionAddress`` represents the public address of a Tor hidden service.
+    A ``TorOnionAddress`` represents the public address of a Tor hidden
+    service.
 
     :ivar type: A string describing the type of transport, 'onion'.
 
@@ -421,9 +478,14 @@ class TorOnionAddress(FancyEqMixin, object):
 
 
 class IHiddenService(Interface):
-    local_address = Attribute('The actual machine address we are listening on.')
-    hidden_service_dir = Attribute('The hidden service directory, where "hostname" and "private_key" files live.')
-    tor_config = Attribute('The TorConfig object attached to the Tor hosting this hidden service (in turn has .protocol for TorControlProtocol).')
+    local_address = Attribute(
+        'The actual machine address we are listening on.')
+    hidden_service_dir = Attribute(
+        'The hidden service directory, where "hostname" and "private_key" '
+        'files live.')
+    tor_config = Attribute(
+        'The TorConfig object attached to the Tor hosting this hidden service '
+        '(in turn has .protocol for TorControlProtocol).')
 
 
 @implementer(IListeningPort, IHiddenService)
@@ -432,11 +494,13 @@ class TorOnionListeningPort(object):
     Our TCPHiddenServiceEndpoint's `listen` method will return a deferred
     which fires an instance of this object.
     The `getHost` method will return a TorOnionAddress instance... which
-    can be used to determine the onion address of a newly created Tor Hidden Service.
+    can be used to determine the onion address of a newly created Tor Hidden
+    Service.
 
-    `startListening` and `stopListening` methods proxy to the "TCP ListeningPort" object...
-    which implements IListeningPort interface but has many more responsibilities we needn't
-    worry about here.
+    `startListening` and `stopListening` methods proxy to the "TCP
+    ListeningPort" object...
+    which implements IListeningPort interface but has many more
+    responsibilities we needn't worry about here.
     """
 
     def __init__(self, listening_port, hs_dir, uri, port, tor_config):
@@ -482,8 +546,8 @@ class TCPHiddenServiceEndpointParser(object):
 
     ``onion:80``
 
-    If ``controlPort`` is specified, it means connect to an already-running Tor on
-    that port and add a hidden-serivce to it.
+    If ``controlPort`` is specified, it means connect to an already-running Tor
+    on that port and add a hidden-serivce to it.
 
     ``localPort`` is optional and if not specified, a port is selected by
     the OS.
@@ -499,7 +563,9 @@ class TCPHiddenServiceEndpointParser(object):
     # we should use camelCase in the endpoint definitions...
     def parseStreamServer(self, reactor, public_port, localPort=None,
                           controlPort=None, hiddenServiceDir=None):
-        ''':api:`twisted.internet.interfaces.IStreamServerEndpointStringParser`'''
+        '''
+        :api:`twisted.internet.interfaces.IStreamServerEndpointStringParser`
+        '''
 
         public_port = int(public_port)
 
@@ -516,7 +582,8 @@ class TCPHiddenServiceEndpointParser(object):
 
         if controlPort:
             try:
-                ep = clientFromString(reactor, "tcp:host=127.0.0.1:port=%d" % int(controlPort))
+                ep = clientFromString(
+                    reactor, "tcp:host=127.0.0.1:port=%d" % int(controlPort))
             except ValueError:
                 ep = clientFromString(reactor, "unix:path=%s" % controlPort)
             return TCPHiddenServiceEndpoint.system_tor(reactor, ep,
@@ -528,3 +595,161 @@ class TCPHiddenServiceEndpointParser(object):
                                                    hidden_service_dir=hsd,
                                                    local_port=localPort,
                                                    control_port=controlPort)
+
+
+def default_tcp4_endpoint_generator(*args, **kw):
+    """
+    Default generator used to create client-side TCP4ClientEndpoint
+    instances.  We do this to make the unit tests work...
+    """
+    return TCP4ClientEndpoint(*args, **kw)
+
+
+@implementer(IStreamClientEndpoint)
+class TorClientEndpoint(object):
+    """
+    I am an endpoint class who attempts to establish a SOCKS5
+    connection with the system tor process. Either the user must pass
+    a SOCKS port into my constructor OR I will attempt to guess the
+    Tor SOCKS port by iterating over a list of ports that tor is
+    likely to be listening on.
+
+    :param host:
+        The hostname to connect to. This of course can be a Tor Hidden
+        Service onion address.
+
+    :param port: The tcp port or Tor Hidden Service port.
+
+    :param _proxy_endpoint_generator: This is used for unit tests.
+
+    :param socks_port:
+       This optional argument lets the user specify which Tor SOCKS
+       port should be used.
+    """
+    # XXX should get these via the control connection, i.e. ask Tor
+    # via GETINFO net/listeners/socks or whatever
+    socks_ports_to_try = [9050, 9150]
+
+    def __init__(self, host, port,
+                 socks_hostname=None, socks_port=None,
+                 socks_username=None, socks_password=None,
+                 _proxy_endpoint_generator=default_tcp4_endpoint_generator):
+        if host is None or port is None:
+            raise ValueError('host and port must be specified')
+
+        self.host = host
+        self.port = port
+        self._proxy_endpoint_generator = _proxy_endpoint_generator
+        self.socks_hostname = socks_hostname
+        self.socks_port = socks_port
+        self.socks_username = socks_username
+        self.socks_password = socks_password
+
+        if self.socks_port is None:
+            self._socks_port_iter = iter(self.socks_ports_to_try)
+            self._socks_guessing_enabled = True
+        else:
+            self._socks_guessing_enabled = False
+
+    def connect(self, protocolfactory):
+        self.protocolfactory = protocolfactory
+
+        if self._socks_guessing_enabled:
+            self.socks_port = self._socks_port_iter.next()
+
+        d = self._try_connect()
+        return d
+
+    def _try_connect(self):
+        self.tor_socks_endpoint = self._proxy_endpoint_generator(
+            reactor,
+            self.socks_hostname,
+            self.socks_port
+        )
+
+        if self.socks_username is None or self.socks_password is None:
+            ep = SOCKS5ClientEndpoint(
+                self.host,
+                self.port,
+                self.tor_socks_endpoint
+            )
+        else:
+            ep = SOCKS5ClientEndpoint(
+                self.host,
+                self.port,
+                self.tor_socks_endpoint,
+                methods=dict(login=(self.socks_username, self.socks_password))
+            )
+
+        d = ep.connect(self.protocolfactory)
+        if self._socks_guessing_enabled:
+            d.addErrback(self._retry_socks_port)
+        return d
+
+    def _retry_socks_port(self, failure):
+        failure.trap(error.ConnectError)
+        try:
+            self.socks_port = self._socks_port_iter.next()
+        except StopIteration:
+            return failure
+        d = self._try_connect()
+        d.addErrback(self._retry_socks_port)
+        return d
+
+
+@implementer(IPlugin, IStreamClientEndpointStringParserWithReactor)
+class TorClientEndpointStringParser(object):
+    """
+    This provides a twisted IPlugin and
+    IStreamClientEndpointsStringParser so you can call
+    :api:`twisted.internet.endpoints.clientFromString
+    <clientFromString>` with a string argument like:
+
+    ``tor:host=timaq4ygg2iegci7.onion:port=80:socksPort=9050``
+
+    ...or simply:
+
+    ``tor:host=timaq4ygg2iegci7.onion:port=80``
+
+    You may also include a username + password. By default, Tor will
+    not put two streams that provided different authentication on the
+    same circuit.
+
+    ``tor:host=torproject.org:port=443:socksUsername=foo:socksPassword=bar``
+
+    If ``socksPort`` is specified, it means only use that port to
+    attempt to proxy through Tor. If unspecified then try some likely
+    socksPorts such as [9050, 9150].
+
+    NOTE that I'm using camelCase variable names in the endpoint
+    string to be consistent with the rest of Twisted's naming (and
+    their endpoint parsers).
+
+    XXX FIXME if there is no Tor instance found at socksPort, we
+    should launch one. Perhaps a separate option? (Should be on by
+    default, though, I think).
+    """
+    prefix = "tor"
+
+    def _parseClient(self, host=None, port=None,
+                     socksHostname=None, socksPort=None,
+                     socksUsername=None, socksPassword=None):
+        if port is not None:
+            port = int(port)
+        if socksHostname is None:
+            socksHostname = '127.0.0.1'
+        if socksPort is not None:
+            socksPort = int(socksPort)
+
+        return TorClientEndpoint(
+            host, port,
+            socks_hostname=socksHostname, socks_port=socksPort,
+            socks_username=socksUsername, socks_password=socksPassword
+        )
+
+    def parseStreamClient(self, *args, **kwargs):
+        # for Twisted 14 and 15 (and more) the first argument is
+        # 'reactor', for older Twisteds it's not
+        if _HAVE_TX_14:
+            return self._parseClient(*args[1:], **kwargs)
+        return self._parseClient(*args, **kwargs)
