@@ -1,4 +1,12 @@
-from zope.interface import implements, Interface, Attribute
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+from __future__ import with_statement
+
+from zope.interface import implementer
+from zope.interface import Interface, Attribute
 
 
 class IStreamListener(Interface):
@@ -52,6 +60,7 @@ class IStreamListener(Interface):
         """
 
 
+@implementer(IStreamListener)
 class StreamListenerMixin(object):
     """
     Implements all of :class:`txtorcon.IStreamListener` with no-op
@@ -59,8 +68,6 @@ class StreamListenerMixin(object):
     of the notifications.
     """
 
-    implements(IStreamListener)
-
     def stream_new(self, stream):
         pass
 
@@ -106,9 +113,7 @@ class IStreamAttacher(Interface):
         also return a Deferred which will callback with the desired
         circuit. In this case, you will probably need to be aware that
         the callback from :meth:`txtorcon.TorState.build_circuit` does
-        NOT call back with a Circuit (just Tor's response of 'EXTEND
-        1234') and any circuit you do return must be in the BUILT
-        state anyway (which the above will not).
+        not wait for the circuit to be in BUILT state.
 
         See :ref:`attach_streams_by_country.py` for a complete
         example of using a Deferred in an IStreamAttacher.
@@ -141,8 +146,8 @@ class ICircuitContainer(Interface):
         is started (not necessarily finished inside Tor).
         """
 
-    ## FIXME do we need an IStreamContainer that Stream instances get?
-    ## (Currently, they get an ICircuitContainer...)
+    # FIXME do we need an IStreamContainer that Stream instances get?
+    # (Currently, they get an ICircuitContainer...)
     def close_stream(stream, **kwargs):
         """
         Close a stream.
@@ -175,7 +180,8 @@ class ICircuitListener(Interface):
 
     def circuit_closed(circuit, **kw):
         """
-        A circuit has been closed cleanly (won't be in controller's list any more).
+        A circuit has been closed cleanly (won't be in controller's list
+        any more).
 
         :param kw:
             A dict of additional args. REASON is alsways included, and
@@ -192,6 +198,7 @@ class ICircuitListener(Interface):
             remote_reason=None, **kw)) lower-case versions of all the
             keys are also provided (pointing to the same -- usually
             UPPERCASE -- strings as the upper-case keys).
+
         """
 
     def circuit_failed(circuit, **kw):
@@ -218,12 +225,12 @@ class ICircuitListener(Interface):
         """
 
 
+@implementer(ICircuitListener)
 class CircuitListenerMixin(object):
     """
     Implements all of ICircuitListener with no-op methods. Subclass
     from this if you don't care about most of the notifications.
     """
-    implements(ICircuitListener)
 
     def circuit_new(self, circuit):
         pass
