@@ -1,3 +1,7 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
 from __future__ import with_statement
 
 from twisted.python import log
@@ -5,21 +9,20 @@ from twisted.internet import defer
 from twisted.internet.interfaces import IProtocolFactory
 from twisted.internet.error import ConnectionDone
 from twisted.protocols.basic import LineOnlyReceiver
-from zope.interface import implements
+
+from zope.interface import implementer
 
 from txtorcon.util import hmac_sha256, compare_via_hash
 from txtorcon.log import txtorlog
 
 from txtorcon.interface import ITorControlProtocol
-from spaghetti import FSM, State, Transition
+from .spaghetti import FSM, State, Transition
 
 import os
 import re
-import types
 import base64
 
 DEFAULT_VALUE = 'DEFAULT'
-DEBUG = False
 
 
 class TorProtocolError(RuntimeError):
@@ -40,6 +43,7 @@ class TorProtocolError(RuntimeError):
         return str(self.code) + ' ' + self.text
 
 
+@implementer(IProtocolFactory)
 class TorProtocolFactory(object):
     """
     Builds TorControlProtocol objects. Implements IProtocolFactory for
@@ -49,8 +53,6 @@ class TorProtocolFactory(object):
     you should supply a password callback.
     """
 
-    implements(IProtocolFactory)
-
     def __init__(self, password_function=lambda: None):
         """
         Builds protocols to talk to a Tor client on the specified
@@ -103,7 +105,7 @@ class Event(object):
         self.callbacks.remove(cb)
 
     def got_update(self, data):
-        #print self.name,"got_update:",data
+        # print self.name,"got_update:",data
         for cb in self.callbacks:
             cb(data)
 
@@ -135,7 +137,7 @@ def parse_keywords(lines, multiline_values=True):
     rtn = {}
     key = None
     value = ''
-    ## FIXME could use some refactoring to reduce code duplication!
+    # FIXME could use some refactoring to reduce code duplication!
     for line in lines.split('\n'):
         if line.strip() == 'OK':
             continue
@@ -143,7 +145,7 @@ def parse_keywords(lines, multiline_values=True):
         if '=' in line and ' ' not in line.split('=', 1)[0]:
             if key:
                 if key in rtn:
-                    if isinstance(rtn[key], types.ListType):
+                    if isinstance(rtn[key], list):
                         rtn[key].append(unquote(value))
                     else:
                         rtn[key] = [rtn[key], unquote(value)]
@@ -165,7 +167,7 @@ def parse_keywords(lines, multiline_values=True):
                 value = value + '\n' + line
     if key:
         if key in rtn:
-            if isinstance(rtn[key], types.ListType):
+            if isinstance(rtn[key], list):
                 rtn[key].append(unquote(value))
             else:
                 rtn[key] = [rtn[key], unquote(value)]
@@ -174,6 +176,7 @@ def parse_keywords(lines, multiline_values=True):
     return rtn
 
 
+@implementer(ITorControlProtocol)
 class TorControlProtocol(LineOnlyReceiver):
     """
     This is the main class that talks to a Tor and implements the "raw"
@@ -185,7 +188,7 @@ class TorControlProtocol(LineOnlyReceiver):
     :meth:`txtorcon.TorState.build_circuit` allows you to build custom
     circuits.
 
-    :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used
+   :meth:`txtorcon.TorControlProtocol.add_event_listener` can be used
     to listen for specific events.
 
     To see how circuit and stream listeners are used, see
@@ -193,8 +196,6 @@ class TorControlProtocol(LineOnlyReceiver):
     wish to add your own stream or circuit listeners.
     """
 
-    implements(ITorControlProtocol)
-
     def __init__(self, password_function=None):
         """
         :param password_function:
@@ -249,17 +250,17 @@ class TorControlProtocol(LineOnlyReceiver):
         See the helper method :func:`txtorcon.build_tor_connection`.
         """
 
-        ## variables related to the state machine
-        self.defer = None               # Deferred we returned for the current command
+        # variables related to the state machine
+        self.defer = None        # Deferred we returned for the current command
         self.response = ''
         self.code = None
-        self.command = None             # currently processing this command
-        self.commands = []              # queued commands
+        self.command = None      # currently processing this command
+        self.commands = []       # queued commands
 
-        ## Here we build up the state machine. Mostly it's pretty
-        ## simply, confounded by the fact that 600's (notify) can come
-        ## at any time AND can be multi-line itself. Luckily, these
-        ## can't be nested, nor can the responses be interleaved.
+        # Here we build up the state machine. Mostly it's pretty
+        # simply, confounded by the fact that 600's (notify) can come
+        # at any time AND can be multi-line itself. Luckily, these
+        # can't be nested, nor can the responses be interleaved.
 
         idle = State("IDLE")
         recv = State("RECV")
@@ -295,23 +296,36 @@ class TorControlProtocol(LineOnlyReceiver):
 
         self.fsm = FSM([recvnotify, idle, recvmulti, recv])
         self.state_idle = idle
-        ## hand-set initial state default start state is first in the
-        ## list; the above looks nice in dotty though
+        # hand-set initial state default start state is first in the
+        # list; the above looks nice in dotty though
         self.fsm.state = idle
-        if DEBUG:
-            self.debuglog = open('txtorcon-debug.log', 'w')
-            with open('fsm.dot', 'w') as fsmfile:
-                fsmfile.write(self.fsm.dotty())
+        self.stop_debug()
 
-    ## see end of file for all the state machine matcher and
-    ## transition methods.
+    def start_debug(self):
+        self.debuglog = open('txtorcon-debug.log', 'w')
+
+    def stop_debug(self):
+        def noop(*args, **kw):
+            pass
+
+        class NullLog(object):
+            write = noop
+            flush = noop
+        self.debuglog = NullLog()
+
+    def graphviz_data(self):
+        return self.fsm.dotty()
+
+    # see end of file for all the state machine matcher and
+    # transition methods.
 
     def get_info_raw(self, *args):
         """
-        Mostly for internal use; gives you the raw string back from
-        the GETINFO command. See :meth:`getinfo <txtorcon.TorControlProtocol.get_info>`
+        Mostly for internal use; gives you the raw string back from the
+        GETINFO command. See :meth:`getinfo
+        <txtorcon.TorControlProtocol.get_info>`
         """
-        info = ' '.join(map(lambda x: str(x), list(args)))
+        info = ' '.join([str(x) for x in list(args)])
         return self.queue_command('GETINFO %s' % info)
 
     def get_info_incremental(self, key, line_cb):
@@ -327,9 +341,10 @@ class TorControlProtocol(LineOnlyReceiver):
                 line_cb(line)
         return self.queue_command('GETINFO %s' % key, strip_ok_and_call)
 
-    ## The following methods are the main TorController API and
-    ## probably the most interesting for users.
+    # The following methods are the main TorController API and
+    # probably the most interesting for users.
 
+    @defer.inlineCallbacks
     def get_info(self, *args):
         """
         Uses GETINFO to obtain informatoin from Tor.
@@ -344,11 +359,19 @@ class TorControlProtocol(LineOnlyReceiver):
 
         :return:
             a ``Deferred`` which will callback with a dict containing
-            the keys you asked for. This just inserts ``parse_keywords``
-            in the callback chain; if you want to avoid the parsing
+            the keys you asked for. If you want to avoid the parsing
             into a dict, you can use get_info_raw instead.
         """
-        return self.get_info_raw(*args).addCallback(parse_keywords)
+        lines = yield self.get_info_raw(*args)
+        rtn = {}
+        key = None
+        for line in lines.split('\n'):
+            if line.split('=', 1)[0] in args:
+                key = line.split('=', 1)[0]
+                rtn[key] = line.split('=', 1)[1]
+            else:
+                rtn[key] = rtn[key] + '\n' + line
+        defer.returnValue(rtn)
 
     def get_conf(self, *args):
         """
@@ -373,7 +396,9 @@ class TorControlProtocol(LineOnlyReceiver):
         otherwise.
         """
 
-        return self.queue_command('GETCONF %s' % ' '.join(args)).addCallback(parse_keywords).addErrback(log.err)
+        d = self.queue_command('GETCONF %s' % ' '.join(args))
+        d.addCallback(parse_keywords).addErrback(log.err)
+        return d
 
     def get_conf_raw(self, *args):
         """
@@ -397,7 +422,7 @@ class TorControlProtocol(LineOnlyReceiver):
             d = defer.Deferred()
             d.errback(RuntimeError("Expected an even number of arguments."))
             return d
-        strargs = map(lambda x: str(x), args)
+        strargs = [str(x) for x in args]
         keys = [strargs[i] for i in range(0, len(strargs), 2)]
         values = [strargs[i] for i in range(1, len(strargs), 2)]
 
@@ -405,7 +430,7 @@ class TorControlProtocol(LineOnlyReceiver):
             if ' ' in s:
                 return '"%s"' % s
             return s
-        values = map(maybe_quote, values)
+        values = [maybe_quote(v) for v in values]
         args = ' '.join(map(lambda x, y: '%s=%s' % (x, y), keys, values))
         return self.queue_command('SETCONF ' + args)
 
@@ -423,8 +448,7 @@ class TorControlProtocol(LineOnlyReceiver):
         return self.queue_command('SIGNAL %s' % nm)
 
     def add_event_listener(self, evt, callback):
-        """
-        :param evt: event name, see also
+        """:param evt: event name, see also
         :var:`txtorcon.TorControlProtocol.events` .keys()
 
         Add a listener to an Event object. This may be called multiple
@@ -435,6 +459,10 @@ class TorControlProtocol(LineOnlyReceiver):
         argument, that is the text collected for the event from the
         tor control protocol.
 
+        For more information on the events supported, see
+        `control-spec section 4.1
+        <https://gitweb.torproject.org/torspec.git/tree/control-spec.txt#n1260>`_
+
         .. note::
             this is a low-level interface; if you want to follow
             circuit or stream creation etc. see TorState and methods
@@ -445,6 +473,7 @@ class TorControlProtocol(LineOnlyReceiver):
         .. todo::
             need an interface for the callback
             show how to tie in Stem parsing if you want
+
         """
 
         if evt not in self.valid_events.values():
@@ -511,25 +540,25 @@ class TorControlProtocol(LineOnlyReceiver):
         self._maybe_issue_command()
         return d
 
-    ## the remaining methods are internal API implementations,
-    ## callbacks and state-tracking methods -- you shouldn't have any
-    ## need to call them.
+    # the remaining methods are internal API implementations,
+    # callbacks and state-tracking methods -- you shouldn't have any
+    # need to call them.
 
     def lineReceived(self, line):
         """
         :api:`twisted.protocols.basic.LineOnlyReceiver` API
         """
 
-        if DEBUG:
-            self.debuglog.write(line + '\n')
-            self.debuglog.flush()
-
+        self.debuglog.write(line + '\n')
+        self.debuglog.flush()
         self.fsm.process(line)
 
     def connectionMade(self):
         "Protocol API"
         txtorlog.msg('got connection, authenticating')
-        self.protocolinfo().addCallback(self._do_authenticate).addErrback(self._auth_failed)
+        d = self.protocolinfo()
+        d.addCallback(self._do_authenticate)
+        d.addErrback(self._auth_failed)
 
     def connectionLost(self, reason):
         "Protocol API"
@@ -570,22 +599,20 @@ class TorControlProtocol(LineOnlyReceiver):
             (d, cmd, cmd_arg) = self.command
             self.defer = d
 
-            if DEBUG:
-                #print "NOTIFY",code,rest
-                self.debuglog.write(cmd + '\n')
-                self.debuglog.flush()
+            self.debuglog.write(cmd + '\n')
+            self.debuglog.flush()
 
-            self.transport.write(cmd + '\r\n')
+            data = cmd + '\r\n'
+            txtorlog.msg("cmd: {}".format(data.strip()))
+            self.transport.write(data.encode('utf8'))
 
     def _auth_failed(self, fail):
         """
         Errback if authentication fails.
         """
 
-        if self.post_bootstrap:
-            self.post_bootstrap.errback(fail)
-            return None
-        return fail
+        self.post_bootstrap.errback(fail)
+        return None
 
     def _safecookie_authchallenge(self, reply):
         """
@@ -596,16 +623,23 @@ class TorControlProtocol(LineOnlyReceiver):
 
         server_hash = base64.b16decode(kw['SERVERHASH'])
         server_nonce = base64.b16decode(kw['SERVERNONCE'])
-        ## FIXME put string in global. or something.
-        expected_server_hash = hmac_sha256("Tor safe cookie authentication server-to-controller hash",
-                                           self.cookie_data + self.client_nonce + server_nonce)
+        # FIXME put string in global. or something.
+        expected_server_hash = hmac_sha256(
+            "Tor safe cookie authentication server-to-controller hash",
+            self.cookie_data + self.client_nonce + server_nonce
+        )
 
         if not compare_via_hash(expected_server_hash, server_hash):
-            raise RuntimeError('Server hash not expected; wanted "%s" and got "%s".' % (base64.b16encode(expected_server_hash),
-                                                                                        base64.b16encode(server_hash)))
+            raise RuntimeError(
+                'Server hash not expected; wanted "%s" and got "%s".' %
+                (base64.b16encode(expected_server_hash),
+                 base64.b16encode(server_hash))
+            )
 
-        client_hash = hmac_sha256("Tor safe cookie authentication controller-to-server hash",
-                                  self.cookie_data + self.client_nonce + server_nonce)
+        client_hash = hmac_sha256(
+            "Tor safe cookie authentication controller-to-server hash",
+            self.cookie_data + self.client_nonce + server_nonce
+        )
         client_hash_hex = base64.b16encode(client_hash)
         return self.queue_command('AUTHENTICATE %s' % client_hash_hex)
 
@@ -621,19 +655,28 @@ class TorControlProtocol(LineOnlyReceiver):
                 kw = parse_keywords(line[5:].replace(' ', '\n'))
                 methods = kw['METHODS'].split(',')
         if not methods:
-            raise RuntimeError("Didn't find AUTH line in PROTOCOLINFO response.")
+            raise RuntimeError(
+                "Didn't find AUTH line in PROTOCOLINFO response."
+            )
 
         if 'SAFECOOKIE' in methods:
             cookie = re.search('COOKIEFILE="(.*)"', protoinfo).group(1)
             self.cookie_data = open(cookie, 'r').read()
             if len(self.cookie_data) != 32:
-                raise RuntimeError("Expected authentication cookie to be 32 bytes, got %d" % len(self.cookie_data))
+                raise RuntimeError(
+                    "Expected authentication cookie to be 32 bytes, got %d" %
+                    len(self.cookie_data)
+                )
             txtorlog.msg("Using SAFECOOKIE authentication", cookie,
                          len(self.cookie_data), "bytes")
             self.client_nonce = os.urandom(32)
 
-            d = self.queue_command('AUTHCHALLENGE SAFECOOKIE %s' % base64.b16encode(self.client_nonce))
-            d.addCallback(self._safecookie_authchallenge).addCallback(self._bootstrap).addErrback(self._auth_failed)
+            cmd = 'AUTHCHALLENGE SAFECOOKIE ' + \
+                  base64.b16encode(self.client_nonce)
+            d = self.queue_command(cmd)
+            d.addCallback(self._safecookie_authchallenge)
+            d.addCallback(self._bootstrap)
+            d.addErrback(self._auth_failed)
             return
 
         elif 'COOKIE' in methods:
@@ -641,22 +684,34 @@ class TorControlProtocol(LineOnlyReceiver):
             with open(cookie, 'r') as cookiefile:
                 data = cookiefile.read()
             if len(data) != 32:
-                raise RuntimeError("Expected authentication cookie to be 32 bytes, got %d" % len(data))
-            txtorlog.msg("Using COOKIE authentication", cookie, len(data), "bytes")
-            self.authenticate(data).addCallback(self._bootstrap).addErrback(self._auth_failed)
+                raise RuntimeError(
+                    "Expected authentication cookie to be 32 "
+                    "bytes, got %d instead." % len(data)
+                )
+            txtorlog.msg("Using COOKIE authentication",
+                         cookie, len(data), "bytes")
+            d = self.authenticate(data)
+            d.addCallback(self._bootstrap)
+            d.addErrback(self._auth_failed)
             return
 
         if self.password_function:
-            passwd = defer.maybeDeferred(self.password_function)
-            passwd.addCallback(self._do_password_authentication).addErrback(self._auth_failed)
+            d = defer.maybeDeferred(self.password_function)
+            d.addCallback(self._do_password_authentication)
+            d.addErrback(self._auth_failed)
             return
 
-        raise RuntimeError("The Tor I connected to doesn't support SAFECOOKIE nor COOKIE authentication and I have no password_function specified.")
+        raise RuntimeError(
+            "The Tor I connected to doesn't support SAFECOOKIE nor COOKIE"
+            " authentication and I have no password_function specified."
+        )
 
     def _do_password_authentication(self, passwd):
         if not passwd:
             raise RuntimeError("No password available.")
-        self.authenticate(passwd).addCallback(self._bootstrap).addErrback(self._auth_failed)
+        d = self.authenticate(passwd)
+        d.addCallback(self._bootstrap)
+        d.addErrback(self._auth_failed)
 
     def _set_valid_events(self, events):
         "used as a callback; see _bootstrap"
@@ -674,11 +729,12 @@ class TorControlProtocol(LineOnlyReceiver):
         callback.
         """
 
-        ## unfortunately I don't see a way to get this from the runing
-        ## tor like the events...so this was taken from some version
-        ## of the control-spec and must be kept up-to-date (or accpet
-        ## any signal name and just wait for the reply?
-        self.valid_signals = ["RELOAD", "DUMP", "DEBUG", "NEWNYM", "CLEARDNSCACHE"]
+        try:
+            self.valid_signals = yield self.get_info('signal/names')
+            self.valid_signals = self.valid_signals['signal/names']
+        except TorProtocolError:
+            self.valid_signals = ["RELOAD", "DUMP", "DEBUG", "NEWNYM",
+                                  "CLEARDNSCACHE"]
 
         self.version = yield self.get_info('version')
         self.version = self.version['version']
@@ -690,13 +746,10 @@ class TorControlProtocol(LineOnlyReceiver):
         yield self.queue_command('USEFEATURE EXTENDED_EVENTS')
 
         self.post_bootstrap.callback(self)
-        self.post_bootstrap = None
         defer.returnValue(self)
 
-    ##
-    ## State Machine transitions and matchers. See the __init__ method
-    ## for a way to output a GraphViz dot diagram of the machine.
-    ##
+    # State Machine transitions and matchers. See the __init__ method
+    # for a way to output a GraphViz dot diagram of the machine.
 
     def _is_end_line(self, line):
         "for FSM"
@@ -714,7 +767,7 @@ class TorControlProtocol(LineOnlyReceiver):
             return False
 
         sl = len(line) > 3 and line[3] == ' '
-        #print "single line?",line,sl
+        # print "single line?",line,sl
         if sl:
             self.code = code
             return True
@@ -782,7 +835,8 @@ class TorControlProtocol(LineOnlyReceiver):
         "for FSM"
         # print "BCAST",line
         if len(line) > 3:
-            if self.code >= 200 and self.code < 300 and self.command and self.command[2] is not None:
+            if self.code >= 200 and self.code < 300 and \
+               self.command and self.command[2] is not None:
                 self.command[2](line[4:])
                 resp = ''
 
@@ -793,7 +847,9 @@ class TorControlProtocol(LineOnlyReceiver):
         self.response = ''
         if self.code >= 200 and self.code < 300:
             if self.defer is None:
-                raise RuntimeError('Got a response, but didn\'t issue a command: "%s"' % resp)
+                raise RuntimeError(
+                    'Got a response, but didn\'t issue a command: "%s"' % resp
+                )
             if resp.endswith('\nOK'):
                 resp = resp[:-3]
             self.defer.callback(resp)
@@ -807,9 +863,11 @@ class TorControlProtocol(LineOnlyReceiver):
         elif self.code is None:
             raise RuntimeError("No code set yet in broadcast response.")
         else:
-            raise RuntimeError("Unknown code in broadcast response %d." % self.code)
+            raise RuntimeError(
+                "Unknown code in broadcast response %d." % self.code
+            )
 
-        ## note: we don't do this for 600-level responses
+        # note: we don't do this for 600-level responses
         self.command = None
         self.code = None
         self.defer = None
