@@ -1,3 +1,10 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import with_statement
+
+import collections
 import datetime
 import os
 import stat
@@ -6,9 +13,11 @@ import warnings
 
 from twisted.python import log
 from twisted.internet import defer
-from twisted.internet.endpoints import TCP4ClientEndpoint, UNIXClientEndpoint
-from twisted.internet.interfaces import IReactorCore, IStreamClientEndpoint
-from zope.interface import implements
+from twisted.internet.endpoints import TCP4ClientEndpoint
+from twisted.internet.endpoints import UNIXClientEndpoint
+from twisted.internet.interfaces import IReactorCore
+from twisted.internet.interfaces import IStreamClientEndpoint
+from zope.interface import implementer
 
 from txtorcon import TorProtocolFactory
 from txtorcon.stream import Stream
@@ -19,9 +28,14 @@ from txtorcon.torcontrolprotocol import parse_keywords
 from txtorcon.log import txtorlog
 from txtorcon.torcontrolprotocol import TorProtocolError
 
-from txtorcon.interface import ITorControlProtocol, IRouterContainer, ICircuitListener
-from txtorcon.interface import ICircuitContainer, IStreamListener, IStreamAttacher
-from spaghetti import FSM, State, Transition
+from txtorcon.interface import ITorControlProtocol
+from txtorcon.interface import IRouterContainer
+from txtorcon.interface import ICircuitListener
+from txtorcon.interface import ICircuitContainer
+from txtorcon.interface import IStreamListener
+from txtorcon.interface import IStreamAttacher
+from .spaghetti import FSM, State, Transition
+from .util import basestring
 
 
 def _build_state(proto):
@@ -93,12 +107,19 @@ def build_tor_connection(connection, build_state=True, wait_for_proto=True,
                         'Endpoint for argument "connection", got %s' %
                         (connection, ))
 
-    d = endpoint.connect(TorProtocolFactory(password_function=password_function))
+    d = endpoint.connect(
+        TorProtocolFactory(
+            password_function=password_function
+        )
+    )
     if build_state:
-        d.addCallback(build_state if callable(build_state) else _build_state)
+        d.addCallback(build_state
+                      if isinstance(build_state, collections.Callable)
+                      else _build_state)
     elif wait_for_proto:
-        d.addCallback(wait_for_proto if callable(wait_for_proto) else
-                      _wait_for_proto)
+        d.addCallback(wait_for_proto
+                      if isinstance(wait_for_proto, collections.Callable)
+                      else _wait_for_proto)
     return d
 
 
@@ -137,7 +158,7 @@ def flags_from_dict(kw):
         return ''
 
     flags = ''
-    for (k, v) in kw.iteritems():
+    for (k, v) in kw.items():
         if v:
             flags += ' ' + str(k)
     # note that we want the leading space if there's at least one
@@ -145,6 +166,10 @@ def flags_from_dict(kw):
     return flags
 
 
+@implementer(ICircuitListener)
+@implementer(ICircuitContainer)
+@implementer(IRouterContainer)
+@implementer(IStreamListener)
 class TorState(object):
     """
     This tracks the current state of Tor using a TorControlProtocol.
@@ -165,17 +190,29 @@ class TorState(object):
     This is also a good example of the various listeners, and acts as
     an :class:`txtorcon.interface.ICircuitContainer` and
     :class:`txtorcon.interface.IRouterContainer`.
+
+    :cvar DO_NOT_ATTACH:
+    Constant to return from an IAttacher indicating you don't want to
+    attach this stream at all.
     """
 
-    implements(ICircuitListener, ICircuitContainer, IRouterContainer,
-               IStreamListener)
+    @classmethod
+    def from_protocol(cls, protocol, **kw):
+        '''
+        Create a new, boot-strapped TorState from a TorControlProtocol
+        instance.
 
-    def __init__(self, protocol, bootstrap=True, write_state_diagram=False):
+        :return: a Deferred that fires with a TorState instance
+        '''
+        state = TorState(protocol, bootstrap=True)
+        return state.post_bootstrap
+
+    def __init__(self, protocol, bootstrap=True):
         self.protocol = ITorControlProtocol(protocol)
-        ## fixme could use protocol.on_disconnect to re-connect; see issue #3
+        # fixme could use protocol.on_disconnect to re-connect; see issue #3
 
-        ## could override these to get your own Circuit/Stream subclasses
-        ## to track these things
+        # could override these to get your own Circuit/Stream subclasses
+        # to track these things
         self.circuit_factory = Circuit
         self.stream_factory = Stream
 
@@ -193,6 +230,7 @@ class TorState(object):
         self.circuits = {}               # keys on id (integer)
         self.streams = {}                # keys on id (integer)
 
+        self.all_routers = set()         # list of unique routers
         self.routers = {}                # keys by hexid (string) and by unique names
         self.routers_by_name = {}        # keys on name, value always list (many duplicate "Unnamed" routers, for example)
         self.routers_by_hash = {}        # keys by hexid (string)
@@ -204,7 +242,7 @@ class TorState(object):
         self.cleanup = None              # see set_attacher
 
         class die(object):
-            __name__ = 'die'             # FIXME? just to ease spagetti.py:82's pain
+            __name__ = 'die'  # FIXME? just to ease spagetti.py:82's pain
 
             def __init__(self, msg):
                 self.msg = msg
@@ -212,66 +250,61 @@ class TorState(object):
             def __call__(self, *args):
                 raise RuntimeError(self.msg % tuple(args))
 
-        def nothing(*args):
-            pass
-
         waiting_r = State("waiting_r")
         waiting_w = State("waiting_w")
         waiting_p = State("waiting_p")
         waiting_s = State("waiting_s")
 
         def ignorable_line(x):
-            return x.strip() == '.' or x.strip() == 'OK' or x[:3] == 'ns/' or x.strip() == ''
+            x = x.strip()
+            return x in ['.', 'OK', ''] or x.startswith('ns/')
 
-        waiting_r.add_transition(Transition(waiting_r, ignorable_line, nothing))
-        waiting_r.add_transition(Transition(waiting_s, lambda x: x[:2] == 'r ', self._router_begin))
-        ## FIXME use better method/func than die!!
-        waiting_r.add_transition(Transition(waiting_r, lambda x: x[:2] != 'r ', die('Expected "r " while parsing routers not "%s"')))
+        waiting_r.add_transition(Transition(waiting_r, ignorable_line, None))
+        waiting_r.add_transition(Transition(waiting_s, lambda x: x.startswith('r '), self._router_begin))
+        # FIXME use better method/func than die!!
+        waiting_r.add_transition(Transition(waiting_r, lambda x: not x.startswith('r '), die('Expected "r " while parsing routers not "%s"')))
 
-        waiting_s.add_transition(Transition(waiting_w, lambda x: x[:2] == 's ', self._router_flags))
-        waiting_s.add_transition(Transition(waiting_s, lambda x: x[:2] == 'a ', self._router_address))
-        waiting_s.add_transition(Transition(waiting_r, ignorable_line, nothing))
-        waiting_s.add_transition(Transition(waiting_r, lambda x: x[:2] != 's ' and x[:2] != 'a ', die('Expected "s " while parsing routers not "%s"')))
-        waiting_s.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', nothing))
+        waiting_s.add_transition(Transition(waiting_w, lambda x: x.startswith('s '), self._router_flags))
+        waiting_s.add_transition(Transition(waiting_s, lambda x: x.startswith('a '), self._router_address))
+        waiting_s.add_transition(Transition(waiting_r, ignorable_line, None))
+        waiting_s.add_transition(Transition(waiting_r, lambda x: not x.startswith('s ') and not x.startswith('a '), die('Expected "s " while parsing routers not "%s"')))
+        waiting_s.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', None))
 
-        waiting_w.add_transition(Transition(waiting_p, lambda x: x[:2] == 'w ', self._router_bandwidth))
-        waiting_w.add_transition(Transition(waiting_r, ignorable_line, nothing))
-        waiting_w.add_transition(Transition(waiting_s, lambda x: x[:2] == 'r ', self._router_begin))  # "w" lines are optional
-        waiting_w.add_transition(Transition(waiting_r, lambda x: x[:2] != 'w ', die('Expected "w " while parsing routers not "%s"')))
-        waiting_w.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', nothing))
+        waiting_w.add_transition(Transition(waiting_p, lambda x: x.startswith('w '), self._router_bandwidth))
+        waiting_w.add_transition(Transition(waiting_r, ignorable_line, None))
+        waiting_w.add_transition(Transition(waiting_s, lambda x: x.startswith('r '), self._router_begin))  # "w" lines are optional
+        waiting_w.add_transition(Transition(waiting_r, lambda x: not x.startswith('w '), die('Expected "w " while parsing routers not "%s"')))
+        waiting_w.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', None))
 
-        waiting_p.add_transition(Transition(waiting_r, lambda x: x[:2] == 'p ', self._router_policy))
-        waiting_p.add_transition(Transition(waiting_r, ignorable_line, nothing))
-        waiting_p.add_transition(Transition(waiting_s, lambda x: x[:2] == 'r ', self._router_begin))  # "p" lines are optional
+        waiting_p.add_transition(Transition(waiting_r, lambda x: x.startswith('p '), self._router_policy))
+        waiting_p.add_transition(Transition(waiting_r, ignorable_line, None))
+        waiting_p.add_transition(Transition(waiting_s, lambda x: x.startswith('r '), self._router_begin))  # "p" lines are optional
         waiting_p.add_transition(Transition(waiting_r, lambda x: x[:2] != 'p ', die('Expected "p " while parsing routers not "%s"')))
-        waiting_p.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', nothing))
+        waiting_p.add_transition(Transition(waiting_r, lambda x: x.strip() == '.', None))
 
         self._network_status_parser = FSM([waiting_r, waiting_s, waiting_w, waiting_p])
-        if write_state_diagram:
-            with open('routerfsm.dot', 'w') as fsmfile:
-                fsmfile.write(self._network_status_parser.dotty())
 
         self.post_bootstrap = defer.Deferred()
         if bootstrap:
-            if self.protocol.post_bootstrap:
-                self.protocol.post_bootstrap.addCallback(self._bootstrap).addErrback(self.post_bootstrap.errback)
-            else:
-                self._bootstrap()
+            self.protocol.post_bootstrap.addCallback(self._bootstrap)
+            self.protocol.post_bootstrap.addErrback(self.post_bootstrap.errback)
 
     def _router_begin(self, data):
         args = data.split()
         self._router = Router(self.protocol)
         self._router.from_consensus = True
-        self._router.update(args[1],         # nickname
-                            args[2],         # idhash
-                            args[3],         # orhash
-                            datetime.datetime.strptime(args[4] + args[5], '%Y-%m-%f%H:%M:%S'),
-                            args[6],         # ip address
-                            args[7],         # ORPort
-                            args[8])         # DirPort
+        self._router.update(
+            args[1],                  # nickname
+            args[2],                  # idhash
+            args[3],                  # orhash
+            args[4] + ' ' + args[5],  # modified (like '%Y-%m-%f %H:%M:%S')
+            args[6],                  # ip address
+            args[7],                  # ORPort
+            args[8],                  # DirPort
+        )
 
         if self._router.id_hex in self.routers:
-            ## FIXME should I do an update() on this one??
+            # FIXME should I do an update() on this one??
             self._router = self.routers[self._router.id_hex]
             return
 
@@ -288,6 +321,7 @@ class TorState(object):
             self.routers[self._router.name] = self._router
         self.routers[self._router.id_hex] = self._router
         self.routers_by_hash[self._router.id_hex] = self._router
+        self.all_routers.add(self._router)
 
     def _router_flags(self, data):
         args = data.split()
@@ -314,29 +348,31 @@ class TorState(object):
     def _bootstrap(self, arg=None):
         "This takes an arg so we can use it as a callback (see __init__)."
 
-        ## update list of routers (must be before we do the
-        ## circuit-status) note that we're feeding each line
-        ## incrementally to a state-machine called
-        ## _network_status_parser, set up in constructor. "ns" should
-        ## be the empty string, but we call _update_network_status for
-        ## the de-duplication of named routers
+        # update list of routers (must be before we do the
+        # circuit-status) note that we're feeding each line
+        # incrementally to a state-machine called
+        # _network_status_parser, set up in constructor. "ns" should
+        # be the empty string, but we call _update_network_status for
+        # the de-duplication of named routers
 
-        ns = yield self.protocol.get_info_incremental('ns/all',
-                                                      self._network_status_parser.process)
+        ns = yield self.protocol.get_info_incremental(
+            'ns/all',
+            self._network_status_parser.process
+        )
         self._update_network_status(ns)
 
-        ## update list of existing circuits
+        # update list of existing circuits
         cs = yield self.protocol.get_info_raw('circuit-status')
         self._circuit_status(cs)
 
-        ## update list of streams
+        # update list of streams
         ss = yield self.protocol.get_info_raw('stream-status')
         self._stream_status(ss)
 
-        ## update list of existing address-maps
+        # update list of existing address-maps
         key = 'address-mappings/all'
         am = yield self.protocol.get_info_raw(key)
-        ## strip addressmappsings/all= and OK\n from raw data
+        # strip addressmappsings/all= and OK\n from raw data
         am = am[len(key) + 1:]
         for line in am.split('\n'):
             if len(line.strip()) == 0:
@@ -348,14 +384,15 @@ class TorState(object):
         entries = yield self.protocol.get_info_raw("entry-guards")
         for line in entries.split('\n')[1:]:
             if len(line.strip()) == 0 or line.strip() == 'OK':
+                # XXX does this ever really happen?
                 continue
             args = line.split()
             (name, status) = args[:2]
             name = name[:41]
 
-            ## this is sometimes redundant, as a missing entry guard
-            ## usually means it won't be in our list of routers right
-            ## now, but just being on the safe side
+            # this is sometimes redundant, as a missing entry guard
+            # usually means it won't be in our list of routers right
+            # now, but just being on the safe side
             if status.lower() != 'up':
                 self.unusable_entry_guards.append(line)
                 continue
@@ -365,10 +402,10 @@ class TorState(object):
             except KeyError:
                 self.unusable_entry_guards.append(line)
 
-        ## in case process/pid doesn't exist and we don't know the PID
-        ## because we own it, we just leave it as 0 (previously
-        ## guessed using psutil, but that only works if there's
-        ## exactly one tor running anyway)
+        # in case process/pid doesn't exist and we don't know the PID
+        # because we own it, we just leave it as 0 (previously
+        # guessed using psutil, but that only works if there's
+        # exactly one tor running anyway)
         try:
             pid = yield self.protocol.get_info_raw("process/pid")
         except TorProtocolError:
@@ -378,9 +415,9 @@ class TorState(object):
             try:
                 pid = parse_keywords(pid)['process/pid']
                 self.tor_pid = int(pid)
-            except KeyError:
+            except:
                 self.tor_pid = 0
-        elif self.protocol.is_owned:
+        if not self.tor_pid and self.protocol.is_owned:
             self.tor_pid = self.protocol.is_owned
 
         self.post_bootstrap.callback(self)
@@ -410,16 +447,16 @@ class TorState(object):
             self.attacher = None
 
         if self.attacher is None:
-            self.undo_attacher()
+            d = self.undo_attacher()
             if self.cleanup:
                 react.removeSystemEventTrigger(self.cleanup)
                 self.cleanup = None
 
         else:
-            self.protocol.set_conf("__LeaveStreamsUnattached", "1")
+            d = self.protocol.set_conf("__LeaveStreamsUnattached", "1")
             self.cleanup = react.addSystemEventTrigger('before', 'shutdown',
                                                        self.undo_attacher)
-        return None
+        return d
 
     stream_close_reasons = {
         'REASON_MISC': 1,               # (catch-all for unlisted reasons)
@@ -447,7 +484,7 @@ class TorState(object):
         """
 
         if type(stream) != int:
-            ## assume it's a Stream instance
+            # assume it's a Stream instance
             stream = stream.id
         try:
             reason = int(reason)
@@ -455,11 +492,13 @@ class TorState(object):
             try:
                 reason = TorState.stream_close_reasons[reason]
             except KeyError:
-                raise ValueError('Unknown stream close reason "%s"' % str(reason))
+                raise ValueError(
+                    'Unknown stream close reason "%s"' % str(reason)
+                )
 
         flags = flags_from_dict(kwargs)
 
-        ## stream is now an ID no matter what we passed in
+        # stream is now an ID no matter what we passed in
         cmd = 'CLOSESTREAM %d %d%s' % (stream, reason, flags)
         return self.protocol.queue_command(cmd)
 
@@ -481,10 +520,12 @@ class TorState(object):
         """
 
         if type(circid) != int:
-            ## assume it's a Circuit instance
+            # assume it's a Circuit instance
             circid = circid.id
         flags = flags_from_dict(kwargs)
-        return self.protocol.queue_command('CLOSECIRCUIT %s%s' % (circid, flags))
+        return self.protocol.queue_command(
+            'CLOSECIRCUIT %s%s' % (circid, flags)
+        )
 
     def add_circuit_listener(self, icircuitlistener):
         listen = ICircuitListener(icircuitlistener)
@@ -531,7 +572,10 @@ class TorState(object):
 
         else:
             if using_guards and routers[0] not in self.entry_guards.values():
-                warnings.warn("Building a circuit not starting with a guard: %s" % (str(routers),), RuntimeWarning)
+                warnings.warn(
+                    "Circuit doesn't start with a guard: %s" % routers,
+                    RuntimeWarning
+                )
             cmd = "EXTENDCIRCUIT 0 "
             first = True
             for router in routers:
@@ -539,7 +583,8 @@ class TorState(object):
                     first = False
                 else:
                     cmd += ','
-                if isinstance(router, types.StringType) and len(router) == 40 and hashFromHexId(router):
+                if isinstance(router, basestring) and len(router) == 40 \
+                   and hashFromHexId(router):
                     cmd += router
                 else:
                     cmd += router.id_hex[1:]
@@ -564,48 +609,81 @@ class TorState(object):
         (neither attaching it, nor telling Tor to attach it).
         """
 
-        if self.attacher:
-            if stream.target_host is not None and '.exit' in stream.target_host:
-                ## we want to totally ignore .exit URIs as these are
-                ## used to specify a particular exit node, and trying
-                ## to do STREAMATTACH on them will fail with an error
-                ## from Tor anyway.
-                txtorlog.msg("ignore attacher:", stream)
-                return
+        if self.attacher is None:
+            return None
 
-            circ = IStreamAttacher(self.attacher).attach_stream(stream, self.circuits)
-            if circ is self.DO_NOT_ATTACH:
-                return
+        if stream.target_host is not None \
+           and '.exit' in stream.target_host:
+            # we want to totally ignore .exit URIs as these are
+            # used to specify a particular exit node, and trying
+            # to do STREAMATTACH on them will fail with an error
+            # from Tor anyway.
+            txtorlog.msg("ignore attacher:", stream)
+            return
 
+        # handle async or sync .attach() the same
+        circ_d = defer.maybeDeferred(
+            self.attacher.attach_stream,
+            stream, self.circuits,
+        )
+
+        # actually do the attachment logic; .attach() can return 3 things:
+        #    1. None: let Tor do whatever it wants
+        #    2. DO_NOT_ATTACH: don't attach the stream at all
+        #    3. Circuit instance: attach to the provided circuit
+        def issue_stream_attach(circ):
+            txtorlog.msg("circuit:", circ)
             if circ is None:
-                self.protocol.queue_command("ATTACHSTREAM %d 0" % stream.id)
+                # tell Tor to do what it likes
+                return self.protocol.queue_command("ATTACHSTREAM %d 0" % stream.id)
+
+            elif circ is self.DO_NOT_ATTACH:
+                # do nothing; don't attach the stream
+                return
 
             else:
-                if isinstance(circ, defer.Deferred):
-                    class IssueStreamAttach:
-                        def __init__(self, state, streamid):
-                            self.stream_id = streamid
-                            self.state = state
+                # should get a Circuit instance; check it for suitability
+                if not isinstance(circ, Circuit):
+                    raise RuntimeError(
+                        "IStreamAttacher.attach() must return a Circuit instance "
+                        "(or None or DO_NOT_ATTACH): %s"
+                    )
+                if circ.id not in self.circuits:
+                    raise RuntimeError(
+                        "Attacher returned a circuit unknown to me."
+                    )
+                if circ.state != 'BUILT':
+                    raise RuntimeError(
+                        "Can only attach to BUILT circuits; %d is in %s." %
+                        (circ.id, circ.state)
+                    )
+                # we've got a valid Circuit instance; issue the command
+                return self.protocol.queue_command(
+                    "ATTACHSTREAM %d %d" % (stream.id, circ.id)
+                )
 
-                        def __call__(self, arg):
-                            circid = arg.id
-                            self.state.protocol.queue_command("ATTACHSTREAM %d %d" % (self.stream_id, circid))
+        circ_d.addCallback(issue_stream_attach)
+        circ_d.addErrback(self._attacher_error)
+        return circ_d
 
-                    circ.addCallback(IssueStreamAttach(self, stream.id)).addErrback(log.err)
+    def _attacher_error(self, fail):
+        """
+        not ideal, but there's not really a good way to let the caller
+        handler errors :/ since we ultimately call this due to an
+        async request from Tor. Mostly these errors will be logic or
+        syntax errors in the caller's code anyway.
 
-                else:
-                    if circ.id not in self.circuits:
-                        raise RuntimeError("Attacher returned a circuit unknown to me.")
-                    if circ.state != 'BUILT':
-                        raise RuntimeError("Can only attach to BUILT circuits; %d is in %s." % (circ.id, circ.state))
-                    self.protocol.queue_command("ATTACHSTREAM %d %d" % (stream.id, circ.id))
+        tests monkey-patch this to reduce spew
+        """
+        print("Failure while attaching stream:", fail)
+        return fail
 
     def _circuit_status(self, data):
         """Used internally as a callback for updating Circuit information"""
 
         data = data[len('circuit-status='):].split('\n')
-        ## sometimes there's a newline after circuit-status= and
-        ## sometimes not, so we get rid of it
+        # sometimes there's a newline after circuit-status= and
+        # sometimes not, so we get rid of it
         if len(data) and len(data[0].strip()) == 0:
             data = data[1:]
 
@@ -636,11 +714,12 @@ class TorState(object):
         from NS and NEWCONSENSUS events.
         """
 
+        self.all_routers = set()
         for line in data.split('\n'):
             self._network_status_parser.process(line)
 
         txtorlog.msg(len(self.routers_by_name), "named routers found.")
-        ## remove any names we added that turned out to have dups
+        # remove any names we added that turned out to have dups
         for (k, v) in self.routers.items():
             if v is None:
                 txtorlog.msg(len(self.routers_by_name[k]), "dups:", k)
@@ -664,7 +743,7 @@ class TorState(object):
         from CIRC events.
         """
 
-        #print "circuit_update",line
+        # print("circuit_update", line)
         args = line.split()
         circ_id = int(args[0])
 
@@ -677,9 +756,9 @@ class TorState(object):
         from STREAM events.
         """
 
-        #print "stream_update",line
+        # print("stream_update", line)
         if line.strip() == 'stream-status=':
-            ## this happens if there are no active streams
+            # this happens if there are no active streams
             return
 
         args = line.split()
@@ -695,9 +774,9 @@ class TorState(object):
             wasnew = True
         self.streams[stream_id].update(args)
 
-        ## if the update closed the stream, it won't be in our list
-        ## anymore. FIXME: how can we ever hit such a case as the
-        ## first update being a CLOSE?
+        # if the update closed the stream, it won't be in our list
+        # anymore. FIXME: how can we ever hit such a case as the
+        # first update being a CLOSE?
         if wasnew and stream_id in self.streams:
             self._maybe_attach(self.streams[stream_id])
 
@@ -719,17 +798,20 @@ class TorState(object):
         """
 
         for (event, func) in self.event_map.items():
-            ## the map contains unbound methods, so we bind them
-            ## to self so they call the right thing
-            yield self.protocol.add_event_listener(event, types.MethodType(func, self, TorState))
+            # the map contains unbound methods, so we bind them
+            # to self so they call the right thing
+            yield self.protocol.add_event_listener(
+                event,
+                types.MethodType(func, self, TorState)
+            )
 
-    ## ICircuitContainer
+    # ICircuitContainer
 
     def find_circuit(self, circid):
         "ICircuitContainer API"
         return self.circuits[circid]
 
-    ## IRouterContainer
+    # IRouterContainer
 
     def router_from_id(self, routerid):
         """IRouterContainer API"""
@@ -754,7 +836,7 @@ class TorState(object):
             self.routers[router.id_hex] = router
             return router
 
-    ## implement IStreamListener
+    # implement IStreamListener
 
     def stream_new(self, stream):
         "IStreamListener: a new stream has been created"
@@ -797,7 +879,7 @@ class TorState(object):
         txtorlog.msg("stream_failed", stream.id)
         del self.streams[stream.id]
 
-    ## implement ICircuitListener
+    # implement ICircuitListener
 
     def circuit_launched(self, circuit):
         "ICircuitListener API"
@@ -810,9 +892,11 @@ class TorState(object):
 
     def circuit_built(self, circuit):
         "ICircuitListener API"
-        txtorlog.msg("circuit_built:", circuit.id,
-                     "->".join("%s.%s" % (x.name, x.location.countrycode) for x in circuit.path),
-                     circuit.streams)
+        txtorlog.msg(
+            "circuit_built:", circuit.id,
+            "->".join("%s.%s" % (x.name, x.location.countrycode) for x in circuit.path),
+            circuit.streams
+        )
 
     def circuit_new(self, circuit):
         "ICircuitListener API"
@@ -822,6 +906,8 @@ class TorState(object):
     def circuit_destroy(self, circuit):
         "Used by circuit_closed and circuit_failed (below)"
         txtorlog.msg("circuit_destroy:", circuit.id)
+        for d in circuit._when_built:
+            d.errback(Exception("Destroying circuit; will never hit BUILT"))
         del self.circuits[circuit.id]
 
     def circuit_closed(self, circuit, **kw):
