@@ -1,3 +1,9 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import with_statement
+
 import glob
 import os
 import hmac
@@ -9,6 +15,7 @@ import struct
 
 from twisted.internet import defer
 from twisted.internet.interfaces import IProtocolFactory
+
 from twisted.internet.endpoints import serverFromString
 
 from zope.interface import implementer
@@ -23,18 +30,28 @@ city = None
 country = None
 asn = None
 
+# XXX probably better to depend on and use "six" for py2/3 stuff?
+try:
+    unicode
+except NameError:
+    py3k = True
+    basestring = str
+else:
+    py3k = False
+    basestring = basestring
+
 
 def create_geoip(fname):
-    ## It's more "pythonic" to just wait for the exception,
-    ## but GeoIP prints out "Can't open..." messages for you,
-    ## which isn't desired here
+    # It's more "pythonic" to just wait for the exception,
+    # but GeoIP prints out "Can't open..." messages for you,
+    # which isn't desired here
     if not os.path.isfile(fname):
         raise IOError("Can't find %s" % fname)
 
     if GeoIP is None:
         return None
 
-    ## just letting any errors make it out
+    # just letting any errors make it out
     return GeoIP.open(fname, GeoIP.GEOIP_STANDARD)
 
 
@@ -44,10 +61,10 @@ def maybe_create_db(path):
     except IOError:
         return None
 
-city, asn, country = map(maybe_create_db,
+city, asn, country = list(map(maybe_create_db,
                          ("/usr/share/GeoIP/GeoLiteCity.dat",
                           "/usr/share/GeoIP/GeoIPASNum.dat",
-                          "/usr/share/GeoIP/IP.dat"))
+                          "/usr/share/GeoIP/GeoIP.dat")))
 
 try:
     import ipaddr as _ipaddr
@@ -81,9 +98,11 @@ def find_tor_binary(globs=('/usr/sbin/', '/usr/bin/',
     # Try to find the tor executable using the shell
     if system_tor:
         try:
-            proc = subprocess.Popen(('which tor'),
-                                    stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                                    shell=True)
+            proc = subprocess.Popen(
+                ('which tor'),
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                shell=True
+            )
         except OSError:
             pass
         else:
@@ -130,7 +149,8 @@ def find_keywords(args, key_filter=lambda x: not x.startswith("$")):
         a dict of key->value (both strings) of all name=value type
         keywords found in args.
     """
-    return dict(x.split('=', 1) for x in args if '=' in x and key_filter(x.split('=')[0]))
+    filtered = [x for x in args if '=' in x and key_filter(x.split('=')[0])]
+    return dict(x.split('=', 1) for x in filtered)
 
 
 def delete_file_or_tree(*args):
